#ifndef AI_H_
#define AI_H_

#include <stdexcept>
#include "renderer.h"
#include "actioneventqueue.h"
#include "common.h"
#include "playerpool.h"
#include "renderer.h"
//#include "timers.h"




typedef struct {
	Uint8 AttackInterval;	//=3		; average delay between computer attacks
	Uint8 AttackDelay;	//=5		; average delay time before computer begins first attack
	float PatrolScan;	//=.016		; minute interval between scanning for enemys while patrolling.
	Uint8 CreditReserve;	//=100		; Structure repair will not begin if available cash falls below this amount.
	float PathDelay;	//=.01		; Delay (minutes) between retrying when path is blocked.
	Uint8 OreNearScan;	//=6		; cell radius to scan when harvesting a single patch of ore
	Uint8 OreFarScan;	//=48		; cells radius to scan when looking for a new ore patch to harvest
	Uint8 AutocreateTime;	//=5		; average minutes between creating an 'autocreate' team
	Uint32 InfantryReserve;	//=3000		; always build infantry if cash reserve is greater than this
	Uint8 InfantryBaseMult;	//=1		; build infantry if building count times this number is less than current infantry quantity (should be more?)
	Uint8 PowerSurplus;	//=50		; build power plants until power surplus is at least this amount
	Uint8 BaseSizeAdd;	//=3		; computer base size can be no larger than the largest human opponent, plus this quantity
	float RefineryRatio;	//=.16		; ratio of base that should be composed of refineries
	Uint8 RefineryLimit;	//=4		; never build more than this many refineries
	float BarracksRatio;	//=.16		; ratio of base that should be composed of barracks
	Uint8 BarracksLimit;	//=2		; never build more than this many barracks
	float WarRatio;		//=.1		; ratio of base that should be composed of war factories
	Uint8 WarLimit;		//=2		; never build more than this many war factories
	float DefenseRatio;	//=.4		; ratio of base that should be defensive structures
	Uint8 DefenseLimit;	//=40		; maximum number of defensive buildings to build
	float AARatio;		//=.14		; ratio of base that should be anti-aircraft defense
	Uint8 AALimit;		//=10		; maximum number of anti-aircraft buildings to build
	float TeslaRatio;	//=.16		; ratio of base that should be telsa coils
	Uint8 TeslaLimit;	//=10		; maximum number of tesla coils to build
	float HelipadRatio;	//=.12		; ratio of base that should be composed of helipads
	Uint8 HelipadLimit;	//=5		; maximum number of helipads to build
	float AirstripRatio;	//=.12		; ratio of base that should be composed of airstrips
	Uint8 AirstripLimit;	//=5		; maximum number of airstrips to build
	bool CompEasyBonus;	//=yes		; When more than one human in game, computer player goes to "easy" mode?
	bool Paranoid;		//=yes		; Will computer players ally with each other if the situation looks bleak?
	Uint8 PowerEmergency;	//=75%		; sell buildings to raise power level if it falls below this percentage
	bool AlwaysAlly;	//		; new setting, force ally with other ai players

}AI_rules;











#define DIFFICUTY_EASY		1
#define DIFFICUTY_MEDIUM	2
#define DIFFICUTY_HARD		3
#define DIFFICUTY_VERYHARD	4

//#define GAME_MODE_SINGLE_PLAYER	0
//#define GAME_MODE_SKIRMISH	    1
//#define GAME_MODE_MULTI_PLAYER  2


#define AI_MISSION_NONE			1	// No mission
#define AI_MISSION_GUARD		2	// Guard a area
#define AI_MISSION_ATTACK_BASE		3	// Attack the base of the human user



class ai{
	public:
		ai();
		~ai();
		void SetDifficulty (int Diff);
		void DefendUnits (Player *Player, int PlayerNumb);
		void DefendComputerPlayerBaseUnderAttack (Player *Player, int PlayerNumb, UnitOrStructure *Enimy, Structure *StructureUnderAttack);
		void DefendComputerPlayerUnitUnderAttack (Player *Player, int PlayerNumb, UnitOrStructure *Enimy, Unit *UnitUnderAttack);
		void handle ();

	private:
		int 				NumbPlayers;
		int				HumanPlayerNumb;
		int				Difficulty;
		std::vector <Uint32>		LastGuideTickCount, LastGuideGatherTickCount, LastBuildTickCount, LastAttackTickCount, LastDefendTickCount, LastHarvestTickCount;
		std::vector <Uint32>		RetryDeploy;
		Uint32				LplayerBasePos;
		std::vector <Uint16>		UnitBuildMultiplexer;
		bool				guide;
		AI_rules			Rules;
		std::vector <UnitOrStructure*>	player_targets;

//		Uint16	getDist(Uint16 pos1, Uint16 pos2);
		void					guideAttack (Player *Player, int PlayerNumb);
		Uint32					getDist(Uint32 pos1, Uint32 pos2);
		bool					CanBuildAt (Uint8 PlayerNumb, const char *structname, Uint32 pos);
		bool					BuildStructure (Player *Player, int PlayerNumb, const char *structname, Uint32 ConstYardPos);
		unsigned int				FindClosesedTiberium (Unit *Unit);
		void					DefendStructures (Player *Player, int PlayerNumb);
		void					Build (Player *Player, int PlayerNumb);
		Unit*					EnemyUnitInRange (int MyPlayerNumb, Unit* MyUnit, int AttackRange = -1 );
		Unit*					EnemyUnitInRange (int MyPlayerNumb, Structure* MyStructure, int AttackRange = -1 );
		Structure*				EnemyStructureInRange (int MyPlayerNumb, Unit* MyUnit, int AttackRange = -1 );
		void					Harvest (Player *Player, int PlayerNumb);

		void					patrolAndAttack (Player *Player, int PlayerNumb);

};
#endif //AI_H_
