// mode: -*- C++ -*-
#ifndef FONT_H
#define FONT_H
#include <string>
#include <vector>
#include "renderer.h"
#include "shpimage.h"

typedef struct{
	std::string 			fontname;
	std::vector<SDL_Rect>	chrdest;
	SDL_Surface				*fontimg;
}TFontImage;

class FontCache {
	public:
		FontCache (void);
		~FontCache (void);
		void Add (TFontImage Image);
		void Add (std::string fontname, SDL_Surface *img,  std::vector<SDL_Rect> chrdest);
		bool Get (std::string fontname, SDL_Surface **fontimg,  std::vector<SDL_Rect> &chrdest);

	private:
		std::vector <TFontImage> font_cache;
};

// Struct for storing font options
struct FontOptions {
	bool use_anitaliasing;
	bool underline;
	bool double_underline;
};

// The font class
class Font : SHPBase {
public:
	Font(const std::string& fontname);
	~Font();
	Uint32 getHeight() const;
	Uint32 calcTextWidth(const std::string& text) const;
	bool GetFontColor (SDL_Color FColor, SDL_Color OrgFntColor, SDL_Color &FntColor);
	bool GetFontColor (SDL_Color FColor, SDL_Color BColor, SDL_Color OrgFntColor, SDL_Color &FntColor);
	void drawCHAR(const char Character);

	void drawText(const std::string& text, SDL_Surface *SrcSurf, Uint32 SrcStartx, Uint32 SrcStarty, SDL_Surface *DestSurf, SDL_Color FGcolor, Uint32 DestStartx, Uint32 DestStarty) ;

	void UseAntiAliasing(bool status);
	void underline (bool status);
	void double_underline (bool status);

	using SHPBase::getFileName;
	void Load(std::string FontName) ;
	void reload(void);
private:
	// vars
	SDL_Color				FontPal[15];
	SDL_Surface 			*fontimg;
	std::vector<SDL_Rect>	chrdest;
	std::string				fontname;
	FontOptions				Options;
};
#endif

