#include <math.h>

#include "common.h"
#include "config.h"
#include "cpsimage.h"
#include "graphicsengine.h"
#include "widgets.h"
#include "font.h"
#include "imagecache.h"
#include "sdllayer.h"

using std::string;

//#define USE_IMAGE_POOL
//#define	FONT	"scorefnt.fnt"	 TD?
//#define	FONT	"12metfnt.fnt"
//#define	FONT	"type.fnt"       RA?
//#define	FONT	"editfnt.fnt"
//#define	FONT	"3point.fnt"
//#define	FONT	"8point.fnt"
//#define	FONT	"led.fnt"
//#define	FONT	"vcr.fnt"
//#define	FONT	"12metfnt.fnt"
//#define	FONT	"6point.fnt"
//#define	FONT	"grad6fnt.fnt"
//#define	FONT	"help.fnt"



//********************************************************************************************************************************
//********************************************************************************************************************************
//********************************************************************************************************************************
//********************************************************************************************************************************

RA_Label::RA_Label():LabelFont("type.fnt")
{
	// Setup some vars
	Checked			= false;
	Width			= 20,
	Heigth			= 20;
	LabelSurface		= NULL;
	LabelText 		= "Uninitialize";
	DrawingSurface		= NULL;
	DrawingWindow		= NULL;
	BackgroundBackup	= NULL;

	// Initialize the display surface pointer
//	SetDrawingSurface (pc::gfxeng->get_SDL_ScreenSurface());

	// Initialize the checkbox color
	LabelFontColor.r = 0;
	LabelFontColor.g = 0;
	LabelFontColor.b = 0xff;
	ColorKeyColor.r = ColorKeyColor.g = ColorKeyColor.b = 0;

	LabelDest.x = LabelDest.y = 0;

}

RA_Label::~RA_Label(){
	if (LabelSurface != NULL)
		SDL_FreeSurface(LabelSurface);

	if (BackgroundBackup != NULL){
		SDL_FreeSurface(BackgroundBackup);
	}

	LabelSurface = NULL;
}

void RA_Label::SetText (const std::string text)
{
	if (LabelText != text){
		if (BackgroundBackup != NULL && DrawingWindow != NULL){
			SDL_BlitSurface(BackgroundBackup, NULL, DrawingWindow->GetWindowSurface (), &LabelDest);
			SDL_FreeSurface(BackgroundBackup);
			BackgroundBackup = NULL;
		}

		LabelText = text;
		recreate = true;
	}
}

// Return the label text
std::string RA_Label::GetText (void)
{
	return LabelText;
}

// Get the text height
Uint32 RA_Label::getHeight(void)
{
	return LabelFont.getHeight();
}

// Get the text width
Uint32 RA_Label::getWidth()
{
	return LabelFont.calcTextWidth(LabelText);
}

// Get the text width for a text text
Uint32 RA_Label::getWidth(const std::string text)
{
	return LabelFont.calcTextWidth(text);
}

void RA_Label::setcolor (SDL_Color RGBcolor)
{
	if (BackgroundBackup != NULL && DrawingWindow != NULL){
		SDL_BlitSurface(BackgroundBackup, NULL, DrawingWindow->GetWindowSurface (), &LabelDest);
		SDL_FreeSurface(BackgroundBackup);
		BackgroundBackup = NULL;
	}

	if ( RGBcolor.r != LabelFontColor.r || RGBcolor.g != LabelFontColor.g || RGBcolor.b != LabelFontColor.b ){
		memcpy (&LabelFontColor, &RGBcolor, sizeof (SDL_Color));
		recreate = true;
	}
}

void RA_Label::setcolor (Uint8 r, Uint8 g, Uint8 b)
{
SDL_Color RGBcolor;

	RGBcolor.r = r;
	RGBcolor.g = g;
	RGBcolor.b = b;

	setcolor (RGBcolor);
}

// set the text color
void RA_Label::setcolor (Uint32 color)
{
SDL_Color RGBcolor;

	if (LabelSurface != NULL){
		SDL_GetRGB( color, pc::gfxeng->get_SDL_ScreenSurface()->format, &RGBcolor.r, &RGBcolor.g, &RGBcolor.b );
		setcolor (RGBcolor);
	}
}

void RA_Label::SetFont(const  std::string FontName)
{
	LabelFont.Load(FontName);
	recreate = true;
}

void RA_Label::UseAntiAliasing(bool status)
{
	LabelFont.UseAntiAliasing(status);
	recreate = true;
}

void RA_Label::underline(bool status)
{
	LabelFont.underline(status);
	recreate = true;
}


void RA_Label::Create (void)
{
SDL_Rect dest;
SDL_Surface	*tmp;

	if (LabelSurface != NULL)
		SDL_FreeSurface(LabelSurface);

	if (BackgroundBackup != NULL && DrawingWindow != NULL){
		SDL_BlitSurface(BackgroundBackup, NULL, DrawingWindow->GetWindowSurface (), &LabelDest);
		SDL_FreeSurface(BackgroundBackup);
		BackgroundBackup = NULL;
	}

	LabelSurface = NULL;

	if (LabelText.length() == 0)
		return;

	Width = LabelFont.calcTextWidth(LabelText) + 4;
	Heigth = LabelFont.getHeight()+4;

	LabelSurface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, Width, Heigth, 16, 0, 0, 0, 0);

	// this is the destination as needed for the new surface
	dest.x = 0;
	dest.y = 0;
	dest.w = Width;
	dest.h = Heigth;

	// Fill the surface with the correct color
	SDL_FillRect(LabelSurface, &dest, SDL_MapRGB( LabelSurface->format, ColorKeyColor.r, ColorKeyColor.g, ColorKeyColor.b ));

	if (DrawingSurface != NULL)
		LabelFont.drawText(LabelText, DrawingSurface, LabelDest.x, LabelDest.y, LabelSurface, LabelFontColor, 0, 0);
	else if (DrawingWindow != NULL){
		LabelFont.drawText(LabelText, DrawingWindow->GetWindowSurface (), LabelDest.x, LabelDest.y, LabelSurface, LabelFontColor, 0, 0);
	}

	SDL_SetColorKey(LabelSurface, SDL_SRCCOLORKEY, 0);

//    SDL_SetAlpha(LabelSurface, SDL_SRCALPHA, 150);

	tmp = SDL_DisplayFormat(LabelSurface);
	SDL_FreeSurface(LabelSurface);
	LabelSurface = tmp;

	recreate = false;
}

void RA_Label::SetDrawingSurface (SDL_Surface *DwgSurface)
{
	if (DrawingSurface != DwgSurface){
		DrawingWindow = NULL;
		DrawingSurface = DwgSurface;
		recreate = true;
	}
}

void RA_Label::SetDrawingWindow(RA_WindowClass *Window)
{
	if (DrawingWindow != Window){
		DrawingWindow = Window;
		DrawingSurface = NULL;
		recreate = true;
	}
}

void RA_Label::Draw(const std::string& text, SDL_Surface *DrawingSurface, SDL_Color Fcolor, int X, int Y)
{
	SetText (text);
	setcolor (Fcolor);
	SetDrawingSurface (DrawingSurface);
	Draw ( X, Y);
}

void RA_Label::Draw(const std::string& text, SDL_Surface *DrawingSurface, int X, int Y)
{
	SetText (text);
	SetDrawingSurface (DrawingSurface);
	Draw ( X, Y);
}

void RA_Label::Draw(SDL_Surface *DrawingSurface, int X, int Y)
{

	SetDrawingSurface (DrawingSurface);
	Draw ( X, Y);
}

void RA_Label::Draw ( int X, int Y)
{
SDL_Rect BackupDest;

	// Recreate the text if we draw it somewhere else
	if (LabelDest.x != X || LabelDest.y != Y)
		recreate = true;

	// Init LabelDest
	LabelDest.x = X;
	LabelDest.y = Y;
	LabelDest.w = Width;
	LabelDest.h = Heigth;

	// Recreate the Label surface if needed
	if (recreate)
		Create ();

	if (LabelSurface == NULL)
		return;

	if (DrawingSurface == NULL && DrawingWindow == NULL)
		return;

	if (DrawingSurface != NULL){
		SDL_BlitSurface(LabelSurface, NULL, DrawingSurface, &LabelDest);
	}else if (DrawingWindow != NULL){
		if (BackgroundBackup == NULL){
			BackgroundBackup = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, LabelDest.w, LabelFont.getHeight(), 16, 0, 0, 0, 0);
			BackupDest.x = 0; BackupDest.y = 0; BackupDest.w = LabelDest.w; BackupDest.h = LabelDest.h;
			SDL_BlitSurface( DrawingWindow->GetWindowSurface (), &LabelDest, BackgroundBackup, &BackupDest);
			SDL_BlitSurface(LabelSurface, NULL, DrawingWindow->GetWindowSurface (), &LabelDest);
		}else{
			SDL_BlitSurface(BackgroundBackup, NULL, DrawingWindow->GetWindowSurface (), &LabelDest);
			SDL_BlitSurface(LabelSurface, NULL, DrawingWindow->GetWindowSurface (), &LabelDest);
		}
	}
}

void RA_Label::Redraw ( void )
{
	Draw ( LabelDest.x, LabelDest.y);
}

//********************************************************************************************************************************
//********************************************************************************************************************************
//********************************************************************************************************************************
//********************************************************************************************************************************
// Constructor
ButtonClass::ButtonClass()
{

	this->ButtonImg_up	= NULL;
	this->ButtonImg_down	= NULL;
	this->ButtonImg_over	= NULL;
	this->ButtonState	= BUTTON_STATE_UP;
	this->WindowToDrawOn	= NULL;
	this->ButtonType	= BUTTON_TYPE_DEFAULT;

	DisplaySurface = pc::gfxeng->get_SDL_ScreenSurface();

	// The default button color is white ;)
	if (DisplaySurface != NULL){
		color	= SDL_MapRGB(DisplaySurface->format, 0xff, 0xff, 0xff);
	}

	// Initialize position and size to something
	SizeAndPosition.x = 0;
	SizeAndPosition.y = 0;
	SizeAndPosition.w = 100;
	SizeAndPosition.h = 100;

	(pc::Config.gamenum == GAME_RA)?setcolor (96, 0, 0):setcolor (0, 100, 0);
	(pc::Config.gamenum == GAME_RA)?setFontColor_up (220, 0, 0):setFontColor_up (0, 200, 0);
	(pc::Config.gamenum == GAME_RA)?setFontColor_down (0xff, 0xff, 0xff):setFontColor_down (0, 255, 0);

	ButtonUpLabel.SetFont("grad6fnt.fnt");
	ButtonDownLabel.SetFont("grad6fnt.fnt");
//	ButtonUpLabel.UseAntiAliasing(false);
//	ButtonDownLabel.UseAntiAliasing(false);
}

// Destructor
ButtonClass::~ButtonClass()
{
	if (this->ButtonImg_up != NULL)
		SDL_FreeSurface(ButtonImg_up);

	if (this->ButtonImg_down != NULL)
		SDL_FreeSurface(ButtonImg_down);

	if (this->ButtonImg_over != NULL)
		SDL_FreeSurface(this->ButtonImg_over);
}

// Set the surface (window) to draw the button on
void ButtonClass::SetDrawingSurface(SDL_Surface *Surface)
{
	if (Surface != NULL)
		DisplaySurface = Surface;
}

// Set the surface (window) to draw the button on
void ButtonClass::SetDrawingWindow(RA_WindowClass *Window)
{
	if (Window != NULL){
		WindowToDrawOn = Window;
		DisplaySurface = WindowToDrawOn->GetWindowSurface ();
	}

}

void ButtonClass::setButtonState (int state)
{
	this->ButtonState = state;
}

int ButtonClass::getButtonState (void)
{
	return this->ButtonState;
}

bool ButtonClass::handleMouseEvent (SDL_Event event)
{
bool OldButtonState;

	if (this->ButtonType == BUTTON_TYPE_EXTERNAL)
		return false;

	OldButtonState = this->ButtonState;

	// Change the button state ;)
	if (event.type == SDL_MOUSEBUTTONDOWN){
		if (this->MouseOver()){
			if (this->ButtonState != BUTTON_STATE_DOWN || ButtonType == BUTTON_TYPE_DEFAULT)
				this->ButtonState = BUTTON_STATE_DOWN;
			else
				this->ButtonState = BUTTON_STATE_UP;
		}
	}else if(event.type == SDL_MOUSEBUTTONUP || !this->MouseOver() && this->ButtonState != BUTTON_STATE_UP){
		if (this->ButtonType == BUTTON_TYPE_DEFAULT || this->ButtonState == BUTTON_STATE_OVER){
			this->ButtonState = BUTTON_STATE_UP;
		}
	}else if (MouseOver() && this->ButtonState != BUTTON_STATE_DOWN){
		this->ButtonState = BUTTON_STATE_OVER;
	}//else if (this->ButtonState == BUTTON_STATE_OVER)
	//	this->ButtonState = BUTTON_STATE_UP;

	if (OldButtonState != this->ButtonState){
		//this->drawbutton ();
		return true;
	}

	return false;
}


void ButtonClass::setFontColor_up (Uint8 red, Uint8 green, Uint8 blue)
{
	this->FontColor_up.r = red;
	this->FontColor_up.g = green;
	this->FontColor_up.b = blue;
}

void ButtonClass::setFontColor_down (Uint8 red, Uint8 green, Uint8 blue)
{
	this->FontColor_down.r = red;
	this->FontColor_down.g = green;
	this->FontColor_down.b = blue;
}
void ButtonClass::setcolor (Uint8 red, Uint8 green, Uint8 blue)
{
	ButtonColor.r = red;
	ButtonColor.g = green;
	ButtonColor.b = blue;
	color = SDL_MapRGB(DisplaySurface->format, red, green, blue);
	LightColor = SDL_MapRGB(DisplaySurface->format, (char) (red*1.3), (char) (green*1.3), (char) (blue*1.3));
	DarkColor = SDL_MapRGB(DisplaySurface->format, (char) (red*0.7), (char) (green*0.7), (char) (blue*0.7));
}

void ButtonClass::setcolor (Uint32 SDL_color)
{
	color = SDL_color;
}

Uint32 ButtonClass::GetColor ()
{
	return color;
}

void ButtonClass::setcolor (SDL_Color Color)
{
/*
	color = SDL_MapRGB(DisplaySurface->format, Color.r, Color.g, Color.b);
	LightColor = SDL_MapRGB(DisplaySurface->format, (char) (Color.r*1.3), (char) (Color.g*1.3), (char) (Color.b*1.3));
	DarkColor = SDL_MapRGB(DisplaySurface->format, (char) (Color.r*0.7), (char) (Color.g*0.7), (char) (Color.b*0.7));
*/
	this->setcolor (Color.r, Color.g, Color.b);
//	printf ("%s line %i: Set button rgb color: %i, %i, %i\n", __FILE__, __LINE__, Color.r, Color.g, Color.b);
}


void ButtonClass::settext (const std::string text)
{
	ButtonText = text;

	ButtonUpLabel.SetText (text);
	ButtonDownLabel.SetText (text);
}

void ButtonClass::setposition (int x, int y)
{
	SizeAndPosition.x = x;
	SizeAndPosition.y = y;
}

void ButtonClass::setsize (int w, int h)
{
	SizeAndPosition.w	= w;
	SizeAndPosition.h	= h;
}

/* Function return true if the mouse is on top of the button */
bool ButtonClass::MouseOver(void)
{
int mx, my;
int WinXpos = 0, WinYpos = 0;

	SDL_GetMouseState(&mx, &my);

	if (WindowToDrawOn != NULL){
		WindowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	if (mx > SizeAndPosition.x && mx < SizeAndPosition.x + SizeAndPosition.w)
		if (my > SizeAndPosition.y && my < SizeAndPosition.y + SizeAndPosition.h)
			return true;
	return false;
}


/* Function return true if the mouse is on top of the button */
bool ButtonClass::NeedsRedraw(void)
{
int mx, my;
int WinXpos = 0, WinYpos = 0;

	SDL_GetMouseState(&mx, &my);

	if (WindowToDrawOn != NULL){
		WindowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	if (mx > SizeAndPosition.x - 10 && mx < SizeAndPosition.x + 10 + SizeAndPosition.w)
		if (my > SizeAndPosition.y - 10 && my < SizeAndPosition.y + SizeAndPosition.h)
			return true;
	return false;
}


void ButtonClass::CreateSurface(const std::string& ButtonText, int Xpos, int Ypos, int Width, int Heigth )
{
	this->settext (ButtonText);
	this->setposition (Xpos, Ypos);
	this->setsize (Width, Heigth);
	this->CreateSurface();
}


void ButtonClass::CreateSurface( void )
{
SDL_Rect	dest;
SDL_Surface * tmp;

//printf ("%s line %i: Start create surfaces\n", __FILE__, __LINE__);

	ButtonUpLabel.setcolor (FontColor_up);
	ButtonDownLabel.setcolor (FontColor_down);

	// this is the destination as needed for the new surface
	dest.x = 0;
	dest.y = 0;
	dest.w = SizeAndPosition.w;
	dest.h = SizeAndPosition.h;

	// Free the button image surface (if needed)
	if (this->ButtonImg_up != NULL)
		SDL_FreeSurface(ButtonImg_up);

	ButtonImg_up = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, SizeAndPosition.w, SizeAndPosition.h, 16, 0, 0, 0, 0);

	tmp = SDL_DisplayFormat(ButtonImg_up);
	SDL_FreeSurface(ButtonImg_up);
	ButtonImg_up = tmp;


	// Fill the surface with the correct color
	SDL_FillRect(ButtonImg_up, &dest, color);

	// draw left and right lines
	dest.w = 2;
	dest.h = SizeAndPosition.h;
	dest.x = 0;
	dest.y = 0;
	SDL_FillRect(ButtonImg_up, &dest, LightColor);
	dest.x = SizeAndPosition.w - 2;
	SDL_FillRect(ButtonImg_up, &dest, DarkColor);


	// draw top and bottum lines
	dest.w = SizeAndPosition.w;
	dest.h = 2;
	dest.x = 0;
	dest.y = 0;
	SDL_FillRect(ButtonImg_up, &dest, LightColor);
	dest.y = SizeAndPosition.h - 2;
	SDL_FillRect(ButtonImg_up, &dest, DarkColor);

	//
	SDL_SetColorKey(this->ButtonImg_up, SDL_SRCCOLORKEY, 0);

	int TextWidth = ButtonUpLabel.getWidth(); //this->ButtonFont.calcTextWidth(ButtonText);
	int TextHeith = ButtonUpLabel.getHeight(); //this->ButtonFont.getHeight();

	tmp = SDL_DisplayFormat(ButtonImg_up);
	SDL_FreeSurface(ButtonImg_up);
	ButtonImg_up = tmp;

	ButtonUpLabel.Draw (ButtonImg_up, SizeAndPosition.w/2 - TextWidth/2, SizeAndPosition.h/2 - TextHeith/2);
//	ButtonFont.drawText(this->ButtonText, ButtonImg_up, FontColor_up, SizeAndPosition.w/2 - TextWidth/2, SizeAndPosition.h/2 - TextHeith/2);



//###############################################################
//
//	FROM HERE WE MAKE THE BUTTON DOWN IMAGE
//
//###############################################################

	// this is the destination as needed for the new surface
	dest.x = 0;
	dest.y = 0;
	dest.w = SizeAndPosition.w;
	dest.h = SizeAndPosition.h;

	// Free the button image surface (if needed)
	if (this->ButtonImg_down != NULL)
		SDL_FreeSurface(ButtonImg_down);

	ButtonImg_down = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, SizeAndPosition.w, SizeAndPosition.h, 16, 0, 0, 0, 0);

	tmp = SDL_DisplayFormat(ButtonImg_down);
	SDL_FreeSurface(ButtonImg_down);
	ButtonImg_down = tmp;

	// Fill the surface with the correct color
	SDL_FillRect(ButtonImg_down, &dest, color);

	// draw left and right lines
	dest.w = 2;
	dest.h = SizeAndPosition.h;
	dest.x = 0;
	dest.y = 0;
	SDL_FillRect(ButtonImg_down, &dest, DarkColor);
	dest.x = SizeAndPosition.w - 2;
	SDL_FillRect(ButtonImg_down, &dest, LightColor);


	// draw top and bottum lines
	dest.w = SizeAndPosition.w;
	dest.h = 2;
	dest.x = 0;
	dest.y = 0;
	SDL_FillRect(ButtonImg_down, &dest, DarkColor);
	dest.y = SizeAndPosition.h - 2;
	SDL_FillRect(ButtonImg_down, &dest, LightColor);

	//
	SDL_SetColorKey(this->ButtonImg_down, SDL_SRCCOLORKEY, 0);

	TextWidth = ButtonUpLabel.getWidth(); //this->ButtonFont.calcTextWidth(ButtonText);
	TextHeith = ButtonUpLabel.getHeight(); //this->ButtonFont.getHeight();

	tmp = SDL_DisplayFormat(ButtonImg_down);
	SDL_FreeSurface(ButtonImg_down);
	ButtonImg_down = tmp;

	ButtonDownLabel.Draw (ButtonImg_down, SizeAndPosition.w/2 - TextWidth/2, SizeAndPosition.h/2 - TextHeith/2);
//	ButtonFont.drawText(this->ButtonText, ButtonImg_down, FontColor_down, SizeAndPosition.w/2 - TextWidth/2, SizeAndPosition.h/2 - TextHeith/2);




//###############################################################
//
//	FROM HERE WE MAKE THE BUTTON OVER IMAGE (BUTTON UP WITH FONTCOLOR FOR BUTTON DOWN)
//
//###############################################################

	// this is the destination as needed for the new surface
	dest.x = 0;
	dest.y = 0;
	dest.w = SizeAndPosition.w;
	dest.h = SizeAndPosition.h;

	// Free the button image surface (if needed)
	if (this->ButtonImg_over != NULL)
		SDL_FreeSurface(this->ButtonImg_over);

	ButtonImg_over = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, SizeAndPosition.w, SizeAndPosition.h, 16, 0, 0, 0, 0);

	tmp = SDL_DisplayFormat(ButtonImg_over);
	SDL_FreeSurface(ButtonImg_over);
	ButtonImg_over = tmp;

	// Fill the surface with the correct color
	SDL_FillRect(ButtonImg_over, &dest, color);

	// draw left and right lines
	dest.w = 2;
	dest.h = SizeAndPosition.h;
	dest.x = 0;
	dest.y = 0;
	SDL_FillRect(ButtonImg_over, &dest, LightColor);
	dest.x = SizeAndPosition.w - 2;
	SDL_FillRect(ButtonImg_over, &dest, DarkColor);


	// draw top and bottum lines
	dest.w = SizeAndPosition.w;
	dest.h = 2;
	dest.x = 0;
	dest.y = 0;
	SDL_FillRect(ButtonImg_over, &dest, LightColor);
	dest.y = SizeAndPosition.h - 2;
	SDL_FillRect(ButtonImg_over, &dest, DarkColor);

	//
	SDL_SetColorKey(this->ButtonImg_over, SDL_SRCCOLORKEY, 0);

	TextWidth = ButtonUpLabel.getWidth(); //this->ButtonFont.calcTextWidth(ButtonText);
	TextHeith = ButtonUpLabel.getHeight(); //this->ButtonFont.getHeight();

	tmp = SDL_DisplayFormat(ButtonImg_over);
	SDL_FreeSurface(ButtonImg_over);
	ButtonImg_over = tmp;

	ButtonDownLabel.Draw (ButtonImg_over, SizeAndPosition.w/2 - TextWidth/2, SizeAndPosition.h/2 - TextHeith/2);
//	ButtonFont.drawText(this->ButtonText, ButtonImg_over, FontColor_down, SizeAndPosition.w/2 - TextWidth/2, SizeAndPosition.h/2 - TextHeith/2);

//printf ("%s line %i: End create surfaces\n", __FILE__, __LINE__);
}

/* Draws the button to the screen*/
void ButtonClass::drawbutton (void)
{
	// this is the destination as needed for the new surface
	if (this->ButtonState == BUTTON_STATE_UP && this->ButtonImg_up != NULL){
		SDL_BlitSurface(this->ButtonImg_up, NULL, DisplaySurface, &SizeAndPosition);
		//printf ("Blit button up, buttontext = %s, DisplaySurface = %i\n", ButtonText.c_str(), (int)display);
	}else if (this->ButtonState == BUTTON_STATE_DOWN && this->ButtonImg_down != NULL){
		SDL_BlitSurface(this->ButtonImg_down, NULL, DisplaySurface, &SizeAndPosition);
		//printf ("Blit button down, buttontext = %s\n", ButtonText.c_str());
	}else if (this->ButtonState == BUTTON_STATE_OVER && this->ButtonImg_over != NULL){
		SDL_BlitSurface(this->ButtonImg_over, NULL, DisplaySurface, &SizeAndPosition);
		//printf ("Blit button over, buttontext = %s\n", ButtonText.c_str());
	}else
		printf ("%s line %i: Unknown buttonstate\n", __FILE__, __LINE__);

}

//##########################################################################################
//
//	Window widget class
//
//###########################################################################################


// Constructor
RA_WindowClass::RA_WindowClass()
{
	WindowSurface	= NULL;
	DisplaySurface	= pc::gfxeng->get_SDL_ScreenSurface();
	PalNr		= 0;

//	this->SetupWindow (Xpos, Ypos, Width, Heigth);
}

// Destructor
RA_WindowClass::~RA_WindowClass()
{
	SDL_FreeSurface(WindowSurface);
}


void RA_WindowClass::setPalette(Uint8 pal)
{
	PalNr = pal;
}

void RA_WindowClass::SetupWindow (int Xpos, int Ypos, int Width, int Heigth)
{
	SizeAndPosition.x = Xpos;
	SizeAndPosition.y = Ypos;
	SizeAndPosition.w = Width;
	SizeAndPosition.h = Heigth;

	SDL_FreeSurface(WindowSurface);

	WindowSurface = NULL; // not really needed here ;)

	WindowSurface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, Width, Heigth, 16, 0, 0, 0, 0);

	if (pc::Config.gamenum == GAME_RA){
/**
		The image cache somehow gets corrupted by this --> the imagecache is destroyed and recreated in maps.cpp
*/

		int scaleq = -1;

		// Load window images
        	try {
			Background	= pc::imgcache->loadImage("dd-bkgnd.shp", scaleq);
		}catch (ImageNotFound&){
			printf ("%s line %i: Image not found\n", __FILE__, __LINE__);
		}

        	try {
			LeftBorder	= pc::imgcache->loadImage("dd-left.shp", scaleq);
		}catch (ImageNotFound&){
			printf ("%s line %i: Image not found\n", __FILE__, __LINE__);
		}

        	try {
			RightBorder	= pc::imgcache->loadImage("dd-right.shp", scaleq);
		}catch (ImageNotFound&){
			printf ("%s line %i: Image not found\n", __FILE__, __LINE__);
		}

        	try {
			TopBorder	= pc::imgcache->loadImage("dd-top.shp", scaleq);
		}catch (ImageNotFound&){
			printf ("%s line %i: Image not found\n", __FILE__, __LINE__);
		}

        	try {
			BottomBorder	= pc::imgcache->loadImage("dd-botm.shp", scaleq);
		}catch (ImageNotFound&){
			printf ("%s line %i: Image not found\n", __FILE__, __LINE__);
		}

        	try {
			Corner		= pc::imgcache->loadImage("dd-crnr.shp", scaleq);
		}catch (ImageNotFound&){
			printf ("%s line %i: Image not found\n", __FILE__, __LINE__);
		}

		this->DrawRaBackground ();
	}
}

void RA_WindowClass::ResizeWindow (int Width, int Heigth)
{
SDL_Surface * tmp;

	SizeAndPosition.w = Width;
	SizeAndPosition.h = Heigth;

	SDL_FreeSurface(WindowSurface);

	WindowSurface = NULL; // not really needed here ;)

	WindowSurface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, Width, Heigth, 16, 0, 0, 0, 0);

	tmp = SDL_DisplayFormat(WindowSurface);
	SDL_FreeSurface(WindowSurface);
	WindowSurface = tmp;

	tmp = SDL_DisplayFormat(WindowSurface);
	SDL_FreeSurface(WindowSurface);
	WindowSurface = tmp;

	if (pc::Config.gamenum == GAME_RA)
		this->DrawRaBackground ();
}


void RA_WindowClass::ChangeWindowPosition (int Xpos, int Ypos)
{
	SizeAndPosition.x = Xpos;
	SizeAndPosition.y = Ypos;
}

void RA_WindowClass::GetWindowPosition (int *Xpos, int *Ypos)
{
	*Xpos = SizeAndPosition.x;
	*Ypos = SizeAndPosition.y;
}

SDL_Surface *RA_WindowClass::GetWindowSurface (void)
{
	return WindowSurface;
}


SDL_Surface *RA_WindowClass::ReadShpImage (char *Name, int ImageNumb, Uint8 palnum)
{
SDL_Surface	*image,
		*shadow;
SHPImage	*TempPic;

	try {
		TempPic = new SHPImage(Name, -1);
	} catch (ImageNotFound&) {
		printf ("%s line %i: Image not found\n", __FILE__, __LINE__);
		return NULL;
	}

//	printf ("%s line %i: %s has %i images\n", __FILE__, __LINE__, Name, TempPic->getNumImg());

	TempPic->getImage(ImageNumb, &image, &shadow, palnum /*1*/);

	delete TempPic;

	SDL_FreeSurface(shadow);

	return image;
}

void RA_WindowClass::DrawRaBackground (void)
{
SDL_Rect	dest;
SDL_Surface	*Image,
		*tmp;
int		topspacing, botspacing;
Uint32		EdgeOffset = 0;


	Uint32 BlackColor = SDL_MapRGB(DisplaySurface->format, 10, 10, 10);

	//
	// Draw the image background
	//
	dest.x = 0;
	dest.y = 0;
	dest.w = WindowSurface->w;
	dest.h = WindowSurface->h;
#ifndef USE_IMAGE_POOL
	if ((Image = this->ReadShpImage ((char*)"dd-bkgnd.shp", 0, PalNr))!=NULL){
#else
	if ((Image = pc::imgcache->getImage( Background, 0 ).image) != NULL){
#endif
		tmp = SDL_DisplayFormat(Image);
#ifndef USE_IMAGE_POOL
		SDL_FreeSurface(Image);
#endif
		Image = tmp;



		SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
		if (WindowSurface->w > Image->w){
			dest.x = Image->w;
			dest.w = WindowSurface->w - Image->w;
			dest.h = WindowSurface->h;
			SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
			if (WindowSurface->h > Image->h){
				dest.y = Image->h;
				SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
			}
		}
		if (WindowSurface->h > Image->h){
			dest.x = 0;
			dest.y = Image->h;
			dest.w = WindowSurface->w;
			dest.h = WindowSurface->h;
			SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
		}
#ifndef USE_IMAGE_POOL
		SDL_FreeSurface(Image);
#endif
	}else
		printf ("%s line %i: Failed to load image\n", __FILE__, __LINE__);

	//###################################################
	//
	// Draw the top edge
	//
	//###################################################

#ifndef USE_IMAGE_POOL
	if ((Image = this->ReadShpImage ((char*)"dd-top.shp", 0))!=NULL){
#else
	if ((Image = pc::imgcache->getImage( TopBorder, 0 ).image) != NULL){
#endif
		dest.x = 0;
		dest.y = 0;
		dest.w = WindowSurface->w;
		dest.h = Image->h;
		topspacing = Image->h;
		SDL_FillRect(WindowSurface, &dest, BlackColor);
		SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
		if (WindowSurface->w > Image->w){
			dest.x = Image->w;
			dest.w = WindowSurface->w - Image->w;
			dest.h = Image->h;
			SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
		}
#ifndef USE_IMAGE_POOL
		SDL_FreeSurface(Image);
#endif
	}

	//###################################################
	//
	// Draw the bottom window border
	//
	//###################################################
#ifndef USE_IMAGE_POOL
	if ((Image = this->ReadShpImage ((char*)"dd-botm.shp", 0))!=NULL){
#else
	if ((Image = pc::imgcache->getImage( BottomBorder, 0 ).image) != NULL){
#endif
		dest.x = 0;
		dest.y = WindowSurface->h - Image->h;
		dest.w = WindowSurface->w;
		dest.h = Image->h;
		botspacing = Image->h;
		SDL_FillRect(WindowSurface, &dest, BlackColor);
		SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
		if (WindowSurface->w > Image->w){
			dest.x = Image->w;
			dest.w = WindowSurface->w - Image->w;
			dest.h = Image->h;
			SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
		}
#ifndef USE_IMAGE_POOL
		SDL_FreeSurface(Image);
#endif
	}

	//###################################################
	//
	// Draw the left edge
	//
	//###################################################
#ifndef USE_IMAGE_POOL
	if ((Image = this->ReadShpImage ((char*)"dd-left.shp", 0))!=NULL){
#else
	if ((Image = pc::imgcache->getImage( LeftBorder, 0 ).image) != NULL){
#endif
		EdgeOffset = Image->w;
		dest.x = 0;
		dest.y = 0;
		dest.w = Image->w;
		dest.h = WindowSurface->h;
		SDL_FillRect(WindowSurface, &dest, BlackColor);
		SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
		if (WindowSurface->w > Image->w){
			dest.x = 0;
			dest.y = Image->h;
			dest.w = Image->w;
			dest.h = WindowSurface->h - Image->h;
			SDL_FillRect(WindowSurface, &dest, BlackColor);
			SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
		}
#ifndef USE_IMAGE_POOL
		SDL_FreeSurface(Image);
#endif
	}

	//###################################################
	//
	// Draw the rigth edge
	//
	//###################################################
#ifndef USE_IMAGE_POOL
	if ((Image = this->ReadShpImage ((char*)"dd-right.shp", 0))!= NULL){
#else
	if ((Image = pc::imgcache->getImage( RightBorder, 0 ).image) != NULL){
#endif
		if (Image == NULL)
			printf ("NULL\n");
		dest.x = WindowSurface->w - Image->w;
		dest.y = 0;
		dest.w = Image->w;
		dest.h = WindowSurface->h;
		SDL_FillRect(WindowSurface, &dest, BlackColor);
		SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
		if (WindowSurface->w > Image->w){
			dest.x = WindowSurface->w - Image->w;
			dest.y = Image->h;
			dest.w = Image->w;
			dest.h = WindowSurface->h - Image->h;
			SDL_FillRect(WindowSurface, &dest, BlackColor);
			SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
		}
#ifndef USE_IMAGE_POOL
		SDL_FreeSurface(Image);
#endif
	}
	//###################################################
	//
	// Draw the left/top corner
	//
	//###################################################
#ifndef USE_IMAGE_POOL
	if ((Image = this->ReadShpImage ((char*)"dd-crnr.shp", 0)) != NULL){
#else
	if ((Image = pc::imgcache->getImage( Corner, 0 ).image) != NULL){
#endif
		dest.x = 0;
		dest.y = 0;
		dest.w = Image->w;
		dest.h = Image->h;
		SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
#ifndef USE_IMAGE_POOL
		SDL_FreeSurface(Image);
#endif
	}

	//###################################################
	//
	// Draw the rigth/top corner
	//
	//###################################################
#ifndef USE_IMAGE_POOL
	if ((Image = this->ReadShpImage ((char*)"dd-crnr.shp", 1)) != NULL){
#else
	if ((Image = pc::imgcache->getImage( Corner, 1 ).image) != NULL){
#endif
		dest.x = WindowSurface->w - Image->w;
		dest.y = 0;
		dest.w = Image->w;
		dest.h = Image->h;
		SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
#ifndef USE_IMAGE_POOL
		SDL_FreeSurface(Image);
#endif
	}

	//###################################################
	//
	// Draw the left/bottum corner
	//
	//###################################################
#ifndef USE_IMAGE_POOL
	if ((Image = this->ReadShpImage ((char*)"dd-crnr.shp", 2)) != NULL){
#else
	if ((Image = pc::imgcache->getImage( Corner, 2 ).image) != NULL){
#endif
		dest.x = 0;
		dest.y = WindowSurface->h - Image->h;
		dest.w = Image->w;
		dest.h = Image->h;
		SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
#ifndef USE_IMAGE_POOL
		SDL_FreeSurface(Image);
#endif
	}

	//###################################################
	//
	// Draw the rigth/bottum corner
	//
	//###################################################
#ifndef USE_IMAGE_POOL
	if ((Image = this->ReadShpImage ((char*)"dd-crnr.shp", 3)) != NULL){
#else
	if ((Image = pc::imgcache->getImage( Corner, 3 ).image) != NULL){
#endif
		dest.x = WindowSurface->w - Image->w;
		dest.y = WindowSurface->h - Image->h;
		dest.w = Image->w;
		dest.h = Image->h;
		SDL_BlitSurface(Image, NULL, WindowSurface, &dest);
#ifndef USE_IMAGE_POOL
		SDL_FreeSurface(Image);
#endif
	}


SDL_Rect	SrcRect;
	//###################################################
	//
	// Draw the left edge decoration
	//
	//###################################################
#ifndef USE_IMAGE_POOL
	if ((Image = this->ReadShpImage ((char*)"dd-edge.shp", 0, PalNr))!=NULL){
#else
	if ((Image = pc::imgcache->getImage( Edge, 3 ).image) != NULL){
#endif
		for (int i = topspacing; i < (WindowSurface->h - botspacing); i+=Image->h){
			dest.x = EdgeOffset;
			dest.y = i;
			dest.w = Image->w;
			dest.h = Image->h;
			//memcpy (&SrcRect, &dest, sizeof(SDL_Rect));

			SrcRect.x = 0;
			SrcRect.y = 0;
			SrcRect.w = Image->w;
			SrcRect.h = Image->h;
			if (dest.y + Image->h > WindowSurface->h - botspacing){
				SrcRect.h = WindowSurface->h - dest.y - botspacing;
			}

			SDL_BlitSurface(Image,&SrcRect, WindowSurface, &dest);
		}
		SDL_FreeSurface(Image);
	}

	//###################################################
	//
	// Draw the rigth edge decoration
	//
	//###################################################
#ifndef USE_IMAGE_POOL
	if ((Image = this->ReadShpImage ((char*)"dd-edge.shp", 1, PalNr))!=NULL){
#else
	if ((Image = pc::imgcache->getImage( Edge, 3 ).image) != NULL){
#endif
		for (int i = topspacing; i < (WindowSurface->h - botspacing); i+=Image->h){
			dest.x = WindowSurface->w - EdgeOffset - Image->w;
			dest.y = i;
			dest.w = Image->w;
			dest.h = Image->h;

			SrcRect.x = 0;
			SrcRect.y = 0;
			SrcRect.w = Image->w;
			SrcRect.h = Image->h;
			if (dest.y + Image->h > WindowSurface->h - botspacing){
				printf ("Here\n");
				SrcRect.h = WindowSurface->h - dest.y - botspacing;
			}

			SDL_BlitSurface(Image, &SrcRect, WindowSurface, &dest);
		}
		SDL_FreeSurface(Image);
	}


}

void RA_WindowClass::SolidFill (Uint8 red, Uint8 green, Uint8 blue)
{

	Uint32 Color = SDL_MapRGB(WindowSurface->format, red, green, blue);
	// Fill the surface with the correct color
	SDL_FillRect(WindowSurface, &SizeAndPosition, Color);
}


void RA_WindowClass::DrawWindow (void)
{
/*
SDL_Rect dest;

	dest.x = 0;
	dest.y = 0;
	dest.w = WindowSurface->w;
	dest.h = WindowSurface->h;
*/
	SDL_BlitSurface(WindowSurface, NULL, DisplaySurface, &SizeAndPosition);

}

ListboxClass::ListboxClass()
{
	ListBoxSurface	= NULL;

	ScrollPos	= 0;
	SelectPos	= 0;
	With		= 350;
	NumbLines	= 8;
	ListBoxType	= LISTBOX_TYPE_NORMAL;
//	ListBoxType	= LISTBOX_TYPE_CHECK;
	DisplaySurface	= pc::gfxeng->get_SDL_ScreenSurface();

	btn_up_down	= 0;
	btn_down_down	= 0;

	for (int i = 0; i < 2; i++){
		btn_up[i]	= NULL;
		btn_down[i]	= NULL;
	}

	//printf ("ListBox class initialized\n");
}

ListboxClass::~ListboxClass()
{
	//printf ("Destructor\n");

	if (ListBoxSurface != NULL)
		SDL_FreeSurface(ListBoxSurface);

	for (int i = 0; i < 2; i++){
		if (btn_up[i] != NULL)
			SDL_FreeSurface(btn_up[i]);

		if (btn_down[i] != NULL)
			SDL_FreeSurface(btn_down[i]);
	}

	for (unsigned int i = 0; i < ListboxLines.size(); i++){
		SDL_FreeSurface(ListboxLines[i].Line);
		SDL_FreeSurface(ListboxLines[i].SelectedLine);
	}

}

// Calculate the slider height in pixels
unsigned int ListboxClass::CalcSliderHeight()
{
unsigned int AvailableHeight;

	AvailableHeight = ListBoxSurface->h - 2*LISTBOX_ARROW_UPDOWN_HEIGTH;

	if (MessageList.size() > (unsigned) NumbLines)
		return (AvailableHeight * NumbLines)/MessageList.size();
	else
		return AvailableHeight;
}

// Calculate the slider scroll Yoffset
unsigned int ListboxClass::CalcSliderYoffset()
{
unsigned int AvailableHeight;

	AvailableHeight = ListBoxSurface->h - 2*LISTBOX_ARROW_UPDOWN_HEIGTH;

	if (MessageList.size() > (unsigned) NumbLines && ScrollPos > 0 ){
		if ( (MessageList.size() - NumbLines) > 0)
			return (AvailableHeight - CalcSliderHeight()) * ScrollPos/(MessageList.size() - NumbLines - 1);
	}
	return 0;
}

// Draw the slider bar to the listbox surface
void ListboxClass::DrawSlider()
{
SDL_Surface	*Slider, *tmp;
SDL_Rect	dest;

	if (ListBoxSurface == NULL)
		return;

	Slider = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, LISTBOX_SCROLLBAR_WITH, CalcSliderHeight(), 16, 0, 0, 0, 0);

	tmp = SDL_DisplayFormat(Slider);
	SDL_FreeSurface(Slider);
	Slider = tmp;

	Uint32 ColBlack		= SDL_MapRGB(DisplaySurface->format, 0, 0, 0);
	Uint32 ColEdgeDark	= SDL_MapRGB(DisplaySurface->format, 65, 0, 0);
	Uint32 ColEdgeLigth	= SDL_MapRGB(DisplaySurface->format, 205, 0, 0);

	// Fill the surface with black
	dest.x = 0;
	dest.y = 0;
	dest.w = Slider->w;
	dest.h = Slider->h;
	SDL_FillRect(Slider, &dest, ColBlack);

	// Top
	dest.x = 0;
	dest.y = 0;
	dest.w = Slider->w;
	dest.h = 1;
	SDL_FillRect(Slider, &dest, ColEdgeLigth);

	// Bottum
	dest.x = 0;
	dest.y = Slider->h - 1;
	dest.w = Slider->w;
	dest.h = 1;
	SDL_FillRect(Slider, &dest, ColEdgeDark);

	// Left
	dest.x = 0;
	dest.y = 0;
	dest.w = 1;
	dest.h = Slider->h;
	SDL_FillRect(Slider, &dest, ColEdgeLigth);

	// Rigth
	dest.x = Slider->w - 1;
	dest.y = 0;
	dest.w = 1;
	dest.h = Slider->h;
	SDL_FillRect(Slider, &dest, ColEdgeDark);


	dest.x = ListBoxSurface->w - LISTBOX_SCROLLBAR_WITH;
	dest.y = LISTBOX_ARROW_UPDOWN_HEIGTH + CalcSliderYoffset();
	dest.w = Slider->w;
	dest.h = Slider->h;

	SDL_SetColorKey(Slider,SDL_SRCCOLORKEY, 0x000000);

	SDL_BlitSurface(Slider, NULL, ListBoxSurface, &dest);
	SDL_FreeSurface(Slider);
}

SDL_Surface *ListboxClass::ReadShpImage (char *Name, int ImageNumb, Uint8 palnum)
{
SDL_Surface	*image,
		*shadow;
SHPImage	*TempPic;

	try {
		TempPic = new SHPImage(Name, -1);
	} catch (ImageNotFound&) {
		printf ("%s line %i: Image not found\n", __FILE__, __LINE__);
		return NULL;
	}

	TempPic->getImage(ImageNumb, &image, &shadow, palnum);

	delete TempPic;

	SDL_FreeSurface(shadow);

	SDL_SetColorKey(image,SDL_SRCCOLORKEY, 0xffffff);

	return image;
}


int ListboxClass::GetSelectionIndex (void)
{
	return (/*ScrollPos + */SelectPos);
}

void ListboxClass::SetType (unsigned int type)
{
	if (type < LISTBOX_TYPE_NORMAL || type > LISTBOX_TYPE_CHECK)
		return;

	ListBoxType	= type;
}

bool ListboxClass::IsChecked (unsigned int line)
{
	if (ListBoxType != LISTBOX_TYPE_CHECK)
		return false;

	if (line >= CheckedList.size())
		return false;

	if (CheckedList[line])
		return true;

	return false;
}


void ListboxClass::AddString (const std::string String)
{
	// Add the text to the list
	MessageList.push_back (String);
	CheckedList.push_back (false);
	Recreate = true;
}

SDL_Surface* ListboxClass::geDrawingSurface(void)
{
	return ListBoxSurface;
}

void ListboxClass::ScrollUp (void)
{
//	if (SelectPos > 0){
//		SelectPos--;
//		Recreate = true;
//	}else
	if (ScrollPos > 0){
		ScrollPos--;
		Recreate = true;
	}
}

void ListboxClass::ScrollDown (void)
{
//	if (SelectPos < NumbLines - 1){
//		SelectPos++;
//		Recreate = true;
//	}else
	if ((Uint16)(ScrollPos + 1) < ((Uint16)MessageList.size() - NumbLines)){
		ScrollPos++;
		Recreate = true;
	}
}

void ListboxClass::HandleInput(SDL_Event event)
{
int MouseXpos, MouseYpos;


	switch (event.type) {
		case SDL_MOUSEBUTTONDOWN:
			//printf ("%s line %i: Entry = %i\n", __FILE__, __LINE__, MouseOver_entry());
			SDL_GetMouseState(&MouseXpos, &MouseYpos);

			// Handle item selection by mouse click
			if (ListBoxType	== LISTBOX_TYPE_NORMAL){
				if ((MouseXpos > Xpos + LISTBOX_BORDER_WITH) && (MouseXpos < (ListBoxSurface->w + Xpos - LISTBOX_SCROLLBAR_WITH - LISTBOX_BORDER_WITH))){
					if ((MouseYpos > Ypos + LISTBOX_BORDER_WITH) && (MouseYpos < (ListBoxSurface->h + Ypos - LISTBOX_BORDER_WITH))){
						Uint16 pos = (MouseYpos - Ypos - LISTBOX_BORDER_WITH) / (FontHeigth+LISTBOX_FONT_LINE_SPACING);
						if ((unsigned)(pos + ScrollPos) < MessageList.size()){
							SelectPos = pos + ScrollPos;
							Recreate = true;
						}
					}
				}
			}else if (ListBoxType	== LISTBOX_TYPE_CHECK){
				if ((MouseXpos > Xpos + LISTBOX_BORDER_WITH) && (MouseXpos < (ListBoxSurface->w + Xpos - LISTBOX_SCROLLBAR_WITH - LISTBOX_BORDER_WITH))){
					if ((MouseYpos > Ypos + LISTBOX_BORDER_WITH) && (MouseYpos < (ListBoxSurface->h + Ypos - LISTBOX_BORDER_WITH))){
						Uint16 pos = (MouseYpos - Ypos - LISTBOX_BORDER_WITH) / (FontHeigth+LISTBOX_FONT_LINE_SPACING);
						if ((unsigned)(pos + ScrollPos) < CheckedList.size()){
							if (CheckedList[pos + ScrollPos] == false)
								CheckedList[pos + ScrollPos] = true;
							else
								CheckedList[pos + ScrollPos] = false;
							Recreate = true;
						}
					}
				}
			}

			// Handle the scrollup button
			if (this->MouseOverArrowDown ()){
				this->ScrollDown ();
				btn_down_down = 1;
			}

			// Handle the scrolldown button
			if (this->MouseOverArrowUp ()){
				this->ScrollUp ();
				btn_up_down = 1;
			}
			break;

		case SDL_MOUSEBUTTONUP:
			if (btn_up_down){
				btn_up_down = 0;
				Recreate = true;
			}
			if (btn_down_down){
				btn_down_down = 0;
				Recreate = true;
			}
			break;

		case SDL_KEYDOWN:

			switch (event.key.keysym.sym){
				case SDLK_BACKSPACE:
				case SDLK_TAB:
				case SDLK_RSHIFT:
				case SDLK_LSHIFT:
					break;
				default:
					break;

			}
			break;

	}
}



bool ListboxClass::MouseOverArrowUp (void)
{
int MouseXpos, MouseYpos;

	if (ListBoxSurface == NULL)
		return false;

	SDL_GetMouseState(&MouseXpos, &MouseYpos);

	if ((MouseXpos > (ListBoxSurface->w + Xpos - LISTBOX_SCROLLBAR_WITH)) && (MouseXpos < (ListBoxSurface->w + Xpos)))
		if ((MouseYpos > Ypos) && (MouseYpos < Ypos + LISTBOX_ARROW_UPDOWN_HEIGTH))
			return true;

	return false;
}

bool ListboxClass::MouseOverArrowDown (void)
{
int MouseXpos, MouseYpos;

	SDL_GetMouseState(&MouseXpos, &MouseYpos);

	if ((MouseXpos > (ListBoxSurface->w + Xpos - LISTBOX_SCROLLBAR_WITH)) && (MouseXpos < (ListBoxSurface->w + Xpos)))
		if ((MouseYpos > Ypos + ListBoxSurface->h - LISTBOX_ARROW_UPDOWN_HEIGTH) && (MouseYpos < Ypos + ListBoxSurface->h))
			return true;

	return false;
}


// Create the listbox surface to draw
void ListboxClass::CreateListBox (void)
{
SDL_Rect	dest;
SDL_Surface	*TempImage;
RA_Label	Label;
int		BorderWidth = LISTBOX_BORDER_WITH,
		Spacing = LISTBOX_FONT_LINE_SPACING;
SDL_Color 	RGB_ColWhite, RGB_ColRed;
SDL_Surface	* tmp;

	Label.SetFont("grad6fnt.fnt");

	// Take care of loading the button surfaces only once
	for (int i = 0; i < 2; i++){
		if (btn_up[i] == NULL){
			btn_up[i] = ReadShpImage ((char*)"btn-up.shp", i, 2);
			tmp = SDL_DisplayFormat(btn_up[i]);
			SDL_FreeSurface(btn_up[i]);
			btn_up[i] = tmp;

			//
			// Make buttons from blue to red
			// Stupid hack, but don't know how else to do this
			//
			if (btn_up[i] != NULL){
				SDL_Color color;
				for (Uint32 x = 0; x < (Uint32) btn_up[i]->w; x++){
					for (Uint32 y = 0; y < (Uint32) btn_up[i]->h; y++){
						get_pixel ( btn_up[i], color, x, y );
						//printf ("Color = [%u:%u:%u]\n", color.r, color.g, color.b);
						color.r = (color.b-40)&0xff;
						color.g = color.b = 0;
						set_pixel (btn_up[i], color, x, y );
					}
				}
			}
		}
		if (btn_down[i] == NULL){
			btn_down[i] = ReadShpImage ((char*)"btn-dn.shp", i, 2);
			tmp = SDL_DisplayFormat(btn_down[i]);
			SDL_FreeSurface(btn_down[i]);
			btn_down[i] = tmp;

			//
			// Make buttons from blue to red
			// Stupid hack, but don't know how else to do this
			//
			if (btn_down[i] != NULL){
				SDL_Color color;
				for (Uint32 x = 0; x < (Uint32) btn_down[i]->w; x++){
					for (Uint32 y = 0; y < (Uint32) btn_down[i]->h; y++){
						get_pixel ( btn_down[i], color, x, y );
						color.r = (color.b-40)&0xff;
						color.g = color.b = 0;
						set_pixel (btn_down[i], color, x, y );
					}
				}
			}
		}
	}

	RGB_ColWhite.r = RGB_ColWhite.g = RGB_ColWhite.b = 255;
	RGB_ColRed.r = 205; RGB_ColRed.g = 0; RGB_ColRed.b = 0;

	this->FontHeigth = Label.getHeight();

	if (ListBoxSurface != NULL)
		SDL_FreeSurface(ListBoxSurface);

	ListBoxSurface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, With, ((2 * BorderWidth) + (this->FontHeigth * NumbLines) + (NumbLines* Spacing)), 16, 0, 0, 0, 0);

	tmp = SDL_DisplayFormat(ListBoxSurface);
	SDL_FreeSurface(ListBoxSurface);
	ListBoxSurface = tmp;

	Uint32 ColBlack 	= SDL_MapRGB(DisplaySurface->format, 1, 1, 1);
	Uint32 ColRed		= SDL_MapRGB(DisplaySurface->format, 98, 0, 0);
	Uint32 ColEdgeRed	= SDL_MapRGB(DisplaySurface->format, 255, 0, 0);

	dest.x = 0;
	dest.y = 0;
	dest.w = ListBoxSurface->w;
	dest.h = ListBoxSurface->h;

	SDL_FillRect(ListBoxSurface, &dest, ColBlack);

	// Draw the background for the slider side
	dest.x = ListBoxSurface->w - LISTBOX_SCROLLBAR_WITH;
	dest.y = 0;
	dest.w = LISTBOX_SCROLLBAR_WITH;
	dest.h = ListBoxSurface->h;
	SDL_FillRect(ListBoxSurface, &dest, ColRed);


	// Draw a rectange around the text area
	dest.x = 0;
	dest.w = ListBoxSurface->w;
	dest.h = 1;
	SDL_FillRect(ListBoxSurface, &dest, ColEdgeRed);
	dest.y = ListBoxSurface->h - 1;
	SDL_FillRect(ListBoxSurface, &dest, ColEdgeRed);
	dest.y = 0;
	dest.w = 1;
	dest.h = ListBoxSurface->h;
	SDL_FillRect(ListBoxSurface, &dest, ColEdgeRed);
	dest.x = ListBoxSurface->w - 1 - LISTBOX_SCROLLBAR_WITH;
	SDL_FillRect(ListBoxSurface, &dest, ColEdgeRed);
	dest.x = ListBoxSurface->w - 1;
	SDL_FillRect(ListBoxSurface, &dest, ColEdgeRed);


	// Draw the scroll up icons
	if (btn_up[btn_up_down] != NULL && btn_down[btn_down_down] != NULL){
		if (btn_up_down > 1)
			btn_up_down = 0 ;
		if (btn_down_down > 1)
			btn_down_down = 0 ;

		dest.x = ListBoxSurface->w - btn_up[btn_up_down]->w;
		dest.y = 0;
		dest.w = btn_up[btn_up_down]->w;
		dest.h = btn_up[btn_up_down]->h;
		SDL_BlitSurface(btn_up[btn_up_down], NULL, ListBoxSurface, &dest);

		// Draw the scroll down icons
		dest.y = ListBoxSurface->h - btn_down[btn_down_down]->h;
		SDL_BlitSurface(btn_down[btn_down_down], NULL, ListBoxSurface, &dest);
	}

	if (MessageList.size() <= 0)
		return;

	if (ScrollPos >= MessageList.size())
		ScrollPos = MessageList.size() - 1;


#if 0
	/// TODO cache all line surface in selected and not selected mode, this will speed up considerably !!
	/// Use ListboxLines to do this ;)
	TempImage = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, ListBoxSurface->w - LISTBOX_SCROLLBAR_WITH  - 2 * BorderWidth/*Font.calcTextWidth(MessageList[i+ScrollPos])*/, this->FontHeigth, 16, 0, 0, 0, 0);

	tmp = SDL_DisplayFormat(TempImage);
	SDL_FreeSurface(TempImage);
	TempImage = tmp;

	// Draw the font images on the listbox surface
	for (Uint16 i = 0; i < NumbLines; i++){

		if ((unsigned)(i+ScrollPos) < MessageList.size()){

			if (ListBoxType	== LISTBOX_TYPE_NORMAL){
				dest.x = 0;
				dest.y = 0;
				dest.w = ListBoxSurface->w - LISTBOX_SCROLLBAR_WITH - 10;
				dest.h = this->FontHeigth;

				if (i == (SelectPos - ScrollPos)){
					// Draw the selection line
					SDL_FillRect(TempImage, &dest, ColRed);
					Font.drawText(MessageList[i+ScrollPos], TempImage, RGB_ColWhite, 5, 0);
				}else{
					SDL_FillRect(TempImage, &dest, ColBlack);
					Font.drawText(MessageList[i+ScrollPos], TempImage, RGB_ColWhite, 5, 0);
				}
			}else if (ListBoxType	== LISTBOX_TYPE_CHECK){
				dest.x = 0;
				dest.y = 0;
				dest.w = ListBoxSurface->w - LISTBOX_SCROLLBAR_WITH - 10;
				dest.h = this->FontHeigth;

				if (CheckedList[i + ScrollPos]){
					// Draw the selection line
					SDL_FillRect(TempImage, &dest, ColRed);
					Font.drawText(MessageList[i+ScrollPos], TempImage, RGB_ColWhite, 5, 0);
				}else{
					SDL_FillRect(TempImage, &dest, ColBlack);
					Font.drawText(MessageList[i+ScrollPos], TempImage, RGB_ColWhite, 5, 0);
				}
			}

			dest.x = BorderWidth;
			dest.w = With - 2*BorderWidth;
			dest.y = (FontHeigth * i) + (Spacing * i) + BorderWidth; //TempImage->h * i;
			dest.h = TempImage->h;
			SDL_BlitSurface(TempImage, NULL, ListBoxSurface, &dest);
		}
	}
	SDL_FreeSurface(TempImage);
#else
ListBoxLine Line;
	// Create the line surfaces (if needed)
	if (ListboxLines.size() < (unsigned) MessageList.size()){

		// First free the previous listboxlines.
		for (unsigned int i = 0; i < ListboxLines.size(); i++){
			SDL_FreeSurface(ListboxLines[i].Line);
			SDL_FreeSurface(ListboxLines[i].SelectedLine);
		}
		ListboxLines.clear();

		// Draw the font images on the listbox surface
		for (Uint16 i = 0; i < MessageList.size(); i++){
			if (ListBoxType	== LISTBOX_TYPE_NORMAL){
				dest.x = 0;
				dest.y = 0;
				dest.w = ListBoxSurface->w - LISTBOX_SCROLLBAR_WITH - 10;
				dest.h = this->FontHeigth;

				// Draw the selection line
				TempImage = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, ListBoxSurface->w - LISTBOX_SCROLLBAR_WITH  - 2 * BorderWidth/*Font.calcTextWidth(MessageList[i+ScrollPos])*/, this->FontHeigth, 16, 0, 0, 0, 0);
				tmp = SDL_DisplayFormat(TempImage);
				SDL_FreeSurface(TempImage);
				TempImage = tmp;
				SDL_FillRect(TempImage, &dest, ColRed);
				Label.Draw(MessageList[i+ScrollPos], TempImage, RGB_ColWhite, 5, 0);
				Line.SelectedLine = TempImage;

				// Create the not selected line surface
				TempImage = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, ListBoxSurface->w - LISTBOX_SCROLLBAR_WITH  - 2 * BorderWidth/*Font.calcTextWidth(MessageList[i+ScrollPos])*/, this->FontHeigth, 16, 0, 0, 0, 0);
				tmp = SDL_DisplayFormat(TempImage);
				SDL_FreeSurface(TempImage);
				TempImage = tmp;
				SDL_FillRect(TempImage, &dest, ColBlack);
				Label.Draw(MessageList[i+ScrollPos], TempImage, RGB_ColRed /*RGB_ColWhite*/, 5, 0);
				Line.Line = TempImage;

				ListboxLines.push_back (Line);
			}else if (ListBoxType	== LISTBOX_TYPE_CHECK){
				dest.x = 0;
				dest.y = 0;
				dest.w = ListBoxSurface->w - LISTBOX_SCROLLBAR_WITH - 10;
				dest.h = this->FontHeigth;


				// Create the selected line surface
				TempImage = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, ListBoxSurface->w - LISTBOX_SCROLLBAR_WITH  - 2 * BorderWidth/*Font.calcTextWidth(MessageList[i+ScrollPos])*/, this->FontHeigth, 16, 0, 0, 0, 0);
				tmp = SDL_DisplayFormat(TempImage);
				SDL_FreeSurface(TempImage);
				TempImage = tmp;
				SDL_FillRect(TempImage, &dest, ColRed);
				Label.Draw(MessageList[i+ScrollPos], TempImage, RGB_ColWhite, 5, 0);
				Line.SelectedLine = TempImage;

				// Create the not selected line surface
				TempImage = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, ListBoxSurface->w - LISTBOX_SCROLLBAR_WITH  - 2 * BorderWidth/*Font.calcTextWidth(MessageList[i+ScrollPos])*/, this->FontHeigth, 16, 0, 0, 0, 0);
				tmp = SDL_DisplayFormat(TempImage);
				SDL_FreeSurface(TempImage);
				TempImage = tmp;
				SDL_FillRect(TempImage, &dest, ColBlack);
				Label.Draw(MessageList[i+ScrollPos], TempImage, RGB_ColRed /*RGB_ColWhite*/, 5, 0);
				Line.Line = TempImage;

				ListboxLines.push_back (Line);
			}
		}
	}

	// Draw the line surfaces
	for (Uint16 i = 0; i < NumbLines; i++){
		if ((unsigned)(i+ScrollPos) < MessageList.size()){

			TempImage = NULL;

			if (i == (SelectPos - ScrollPos)){
				// Draw the selection line
				TempImage = ListboxLines[i+ScrollPos].SelectedLine;
			}else{
				// Draw the not selection line
				TempImage = ListboxLines[i+ScrollPos].Line;
			}

			if (TempImage != NULL){
				dest.x = BorderWidth;
				dest.w = With - 2*BorderWidth;
				dest.y = (FontHeigth * i) + (Spacing * i) + BorderWidth; //TempImage->h * i;
				dest.h = TempImage->h;
				SDL_BlitSurface(TempImage, NULL, ListBoxSurface, &dest);
			}
		}
	}
#endif
	DrawSlider();

	Recreate = false;
}


void ListboxClass::DrawListBox (int X, int Y)
{
SDL_Rect dest;

	Xpos = X;
	Ypos = Y;

	if (Recreate)
		this->CreateListBox ();

	if (ListBoxSurface == NULL)
		return;

	dest.x = Xpos;
	dest.y = Ypos;
	dest.w = With;
	dest.h = FontHeigth * NumbLines;

	if (DisplaySurface != NULL)
		SDL_BlitSurface(ListBoxSurface, NULL, DisplaySurface, &dest);
}
//********************************************************************************************************************************
//********************************************************************************************************************************
//********************************************************************************************************************************
//********************************************************************************************************************************

TCheckBox::TCheckBox()
{
	// Setup some vars
	Checked		= false;
	Width		= 20,
	Heigth		= 20;
	CheckBoxSurface = NULL;

	// Initialize the surfaces
	DisplaySurface	= pc::gfxeng->get_SDL_ScreenSurface();

	// Initialize the checkbox color
	CheckboxColor		= SDL_MapRGB(DisplaySurface->format, 0xff, 0, 0);
	CheckboxBackgroundColor	= SDL_MapRGB(DisplaySurface->format, 0, 0, 0);

	this->Create ();
}

TCheckBox::~TCheckBox(){
	SDL_FreeSurface(CheckBoxSurface);
}

void TCheckBox::Create (void)
{
SDL_Rect	dest;

	if (CheckBoxSurface != NULL)
		SDL_FreeSurface(CheckBoxSurface);

	CheckBoxSurface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, Width, Heigth, 16, 0, 0, 0, 0);

	// this is the destination as needed for the new surface
	dest.x = 0;
	dest.y = 0;
	dest.w = Width;
	dest.h = Heigth;

	// Fill the surface with the correct color
	SDL_FillRect(CheckBoxSurface, &dest, CheckboxColor);

	// this is the destination as needed for the new surface
	dest.x = 1;
	dest.y = 1;
	dest.w = Width-2;
	dest.h = Heigth-2;

	// Fill the surface with the correct color
	SDL_FillRect(CheckBoxSurface, &dest, CheckboxBackgroundColor);
}


void TCheckBox::Draw(int X, int Y)
{
SDL_Rect dest;

	if (CheckBoxSurface == NULL)
		return;

	if (DisplaySurface == NULL)
		return;

//	printf ("Draw checkbox\n");

	dest.x = X;
	dest.y = Y;
	dest.w = Width;
	dest.h = Heigth;

	SDL_BlitSurface(CheckBoxSurface, NULL, DisplaySurface, &dest);
}

//********************************************************************************************************************************
//********************************************************************************************************************************
//********************************************************************************************************************************
//********************************************************************************************************************************

TTextBox::TTextBox()
{
	// Setup some vars
	Selected	= false;
	HasMarker   = false;
	Recreate	= true;
	TextBoxSurface	= NULL;
	DisplaySurface	= NULL;
	WindowToDrawOn	= NULL;

	SizeAndPosition.x = 0;
	SizeAndPosition.y = 0;
	SizeAndPosition.h = 16;
	SizeAndPosition.w = 100;

	// Initialize the surfaces
	DisplaySurface	= pc::gfxeng->get_SDL_ScreenSurface();
	//TextBoxSurface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, Width, Heigth, 16, 0, 0, 0, 0);

	// Initialize the checkbox color
	TextColor		        = SDL_MapRGB(DisplaySurface->format, 0xff, 0, 0);
	TextBoxColor		    = SDL_MapRGB(DisplaySurface->format, 0xff, 0, 0);
	TextBoxBackgroundColor	= SDL_MapRGB(DisplaySurface->format, 0, 0, 0);
	TextColorSelected       = SDL_MapRGB(DisplaySurface->format, 255,255,255); //    TextColorNotSelected
	SDL_EnableUNICODE(true);

	// Initialize the text
	TextBoxString = "Player";
}

TTextBox::~TTextBox()
{

	if (TextBoxSurface != NULL)
		SDL_FreeSurface(TextBoxSurface);
}

Uint32 TTextBox::GetWidth(void)
{
 return SizeAndPosition.w;
}

// Set the surface (window) to draw the button on
void TTextBox::SetDrawingWindow(RA_WindowClass *Window)
{
	if (Window != NULL)
		WindowToDrawOn = Window;
}

bool TTextBox::need_redraw(void)
{
	return Recreate;
}

void TTextBox::Create (void)
{
    SDL_Rect	dest;
    Uint16		Ypos;
    char		*temp;
	Recreate	= false;

	TextBoxLabel.SetText(TextBoxString);
	TextBoxLabel.setcolor (TextColor);

	temp = (char *) TextBoxString.c_str();

//	printf ("%s line %i: Recreate text box, text = %s\n", __FILE__, __LINE__, temp);

	if (TextBoxSurface != NULL){
		//printf ("Free surface\n");
		SDL_FreeSurface(TextBoxSurface);
	}

	TextBoxSurface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, SizeAndPosition.w, SizeAndPosition.h, 16, 0, 0, 0, 0);

	SDL_Surface *tmp = SDL_DisplayFormat(TextBoxSurface);
	SDL_FreeSurface(TextBoxSurface);
	TextBoxSurface = tmp;

	// this is the destinatc_strion as needed for the new surface
	dest.x = 0;
	dest.y = 0;
	dest.w = SizeAndPosition.w;
	dest.h = SizeAndPosition.h;

	// Fill the surface with the correct color
	SDL_FillRect(TextBoxSurface, &dest, TextBoxColor);

	// this is the destination as needed for the new surface
	dest.x = 1;
	dest.y = 1;
	dest.w = SizeAndPosition.w-2;
	dest.h = SizeAndPosition.h-2;

	// Fill the surface with the correct color
	SDL_FillRect(TextBoxSurface, &dest, TextBoxBackgroundColor);

	if ((unsigned) SizeAndPosition.h < (unsigned) TextBoxLabel.getHeight())
		return;

	if ((unsigned)  SizeAndPosition.w-8 < (unsigned) TextBoxLabel.getWidth())
		return;

	Ypos = (SizeAndPosition.h - TextBoxLabel.getHeight())/2;

//	TextBoxFont.drawText(TextBoxString, TextBoxSurface, TextColor, 4, Ypos) ; //drawText("Your color", ListBox.geDrawingSurface(), 100, 20);
	TextBoxLabel.Draw(TextBoxSurface, 4, Ypos);

}

bool TTextBox::setText(const std::string& newtext)
{
	if ((unsigned) SizeAndPosition.h-2 < (unsigned) TextBoxLabel.getHeight())
		return false;

	if ((unsigned)  SizeAndPosition.w-8 < (unsigned) TextBoxLabel.getWidth(newtext))
		return false;

	TextBoxString = newtext;
	Recreate = true;

	return true;
}

void TTextBox::setcolor (Uint32 SDL_color)
{
	TextColor = SDL_color;
	TextBoxLabel.setcolor (SDL_color);
}

char *TTextBox::getText(void)
{
     if(HasMarker)
       this->DeleteChar(TextBoxString.size() - 1);

	 return (char*)TextBoxString.c_str();
}


bool TTextBox::AddChar(char AddChar)
{
    std::string	newtext;
	newtext = TextBoxString;
	newtext += AddChar;

	if ((unsigned) SizeAndPosition.h < (unsigned) TextBoxLabel.getHeight())
		return false;

	if ((unsigned)  SizeAndPosition.w-8 < (unsigned) TextBoxLabel.getWidth(newtext))
		return false;

	TextBoxString = newtext;

	Recreate = true;

	return true;
}

bool TTextBox::DeleteChar(unsigned int pos)
{
	if (TextBoxString.size() > pos){
		TextBoxString.erase (pos, 1);
	}else
		return false;

	Recreate = true;

	return true;
}


/* Function return true if the mouse is on top of the button */
bool TTextBox::MouseOver(void)
{
int mx, my;
int WinXpos = 0, WinYpos = 0;

	SDL_GetMouseState(&mx, &my);

	if (WindowToDrawOn != NULL){
		WindowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	if (mx > SizeAndPosition.x && mx < SizeAndPosition.x + SizeAndPosition.w)
		if (my > SizeAndPosition.y && my < SizeAndPosition.y + SizeAndPosition.h)
			return true;
	return false;
}


void TTextBox::HandleInput(SDL_Event event)
{
    //SDL_Event event;
    int mx, my;
    int WinXpos = 0, WinYpos = 0;

	SDL_GetMouseState(&mx, &my);

	if (WindowToDrawOn != NULL){
		WindowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	switch (event.type) {
		case SDL_MOUSEBUTTONDOWN:
			if (MouseOver()){
                if(!Selected){
                  this->AddChar('_');
                  HasMarker = true;
                }
                TextColorNotSelected = TextColor;
                TextColor = TextColorSelected;
				Selected = true;
			}else{
               if(HasMarker){
                  this->DeleteChar(TextBoxString.size() - 1);
                  HasMarker = false;
                  TextColor = TextColorNotSelected;
               }
				Selected = false;
			}
			break;

		case SDL_MOUSEBUTTONUP:
			break;

		case SDL_KEYDOWN:
			if (!Selected)
				return;

			switch (event.key.keysym.sym){
				case SDLK_BACKSPACE:
					// Remove last char
					if(TextBoxString.size() > 1)
					  this->DeleteChar(TextBoxString.size() - 2);
					break;
				case SDLK_TAB:
				case SDLK_RSHIFT:
				case SDLK_LSHIFT:
					break;
				default:
					//SDL_EnableUNICODE(true); is needed for this to work ;)
					char ch = 0;
					if ( (event.key.keysym.unicode & 0xFF80) == 0 ) {
  						ch = event.key.keysym.unicode & 0x7F;
						//printf ("Character = %c\n", ch);
						if ((ch >= '0' && ch <= '9') || (ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z')){
                          if(HasMarker && TextBoxString.size() > 0){
                            this->DeleteChar(TextBoxString.size() - 1);
                            HasMarker = false;
                          }
							this->AddChar (ch);
                            if(!HasMarker){
                              this->AddChar('_');
                              HasMarker = true;
                            }
                        }
                    }
					break;
			}
			break;
	}
}

bool TTextBox::Draw(int X, int Y)
{
	// Set size and posisiton
	SizeAndPosition.x = X;
	SizeAndPosition.y = Y;

	if (this->Recreate)
		Create();

	if (TextBoxSurface == NULL)
		return false;

	if (DisplaySurface == NULL)
		return false;

	if (WindowToDrawOn != NULL)
		SDL_BlitSurface(TextBoxSurface, NULL, WindowToDrawOn->GetWindowSurface (), &SizeAndPosition);
	else
		SDL_BlitSurface(TextBoxSurface, NULL, DisplaySurface, &SizeAndPosition);

	return true;
}

//********************************************************************************************************************************
//********************************************************************************************************************************
//********************************************************************************************************************************
//********************************************************************************************************************************
DropDownListBox::DropDownListBox (){
	List.empty();
	ListBoxSurface	= NULL;
	DisplaySurface	= NULL;
	WindowToDrawOn	= NULL;
	ArrowDownImage	= NULL;

	SizeAndPosition.x = 0;
	SizeAndPosition.y = 0;
	SizeAndPosition.w = 100;
	SizeAndPosition.h = 16;

	SelectedIndex	= 0;
	Selected		= false;
	Recreate		= true;
	button_down		= false;

	// Initialize the surfaces
	DisplaySurface	= pc::gfxeng->get_SDL_ScreenSurface();

	// Initialize the checkbox color
	ListBoxColor			= SDL_MapRGB(DisplaySurface->format, 0xff, 0, 0);
	ListBoxBackgroundColor	= SDL_MapRGB(DisplaySurface->format, 0, 0, 0);

	TextColor.r	=	0xff;
	TextColor.g	=	0;
	TextColor.b	=	0;

	// Load window images
	try {
		ArrowDwn = pc::imgcache->loadImage("btn-dn.shp", 1);
	}catch (ImageNotFound&){
		printf ("%s line %i: Image not found\n", __FILE__, __LINE__);
	}

	printf (" Cache entry = %i\n", ArrowDwn);
}

DropDownListBox::~DropDownListBox()
{
	if (ListBoxSurface != NULL)
		SDL_FreeSurface( ListBoxSurface );

#ifndef USE_IMAGE_POOL
	if (ArrowDownImage != NULL)
		SDL_FreeSurface( ArrowDownImage );
#endif
}


SDL_Surface *DropDownListBox::ReadShpImage (char *Name, int ImageNumb, Uint8 palnum)
{
SDL_Surface	*image,
			*shadow;
SHPImage	*TempPic;

	try {
		TempPic = new SHPImage(Name, -1);
	} catch (ImageNotFound&) {
		printf ("%s line %i: Image not found\n", __FILE__, __LINE__);
		return NULL;
	}

	TempPic->getImage(ImageNumb, &image, &shadow, palnum);

	delete TempPic;

	SDL_FreeSurface(shadow);

	SDL_SetColorKey(image,SDL_SRCCOLORKEY, 0xffffff);

	return image;
}

// Set the surface (window) to draw the button on
void DropDownListBox::SetDrawingWindow(RA_WindowClass *Window)
{
	if (Window != NULL){
		WindowToDrawOn = Window;
	}
}

bool DropDownListBox::need_redraw(void)
{
	return Recreate;
}


// Set the surface (window) to draw the button on
void DropDownListBox::AddEntry(std::string Entry)
{
	List.push_back(Entry);
}


//
// Moet gebruik gaan maken van: btn-dn.shp and btn-up.shp
void DropDownListBox::Create (void)
{
SDL_Rect	dest;
Uint16		Ypos;
//char		*temp;


	Recreate	= false;

	if (ListBoxSurface != NULL){
		//printf ("Free surface\n");
		SDL_FreeSurface(ListBoxSurface);
	}

	Spacing = (SizeAndPosition.h - ListBoxLabel.getHeight())/2;


	if (Selected){
		ListBoxSurface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, SizeAndPosition.w, (SizeAndPosition.h + Spacing + (SizeAndPosition.h - Spacing)* List.size() ), 16, 0, 0, 0, 0);
	}else{
		ListBoxSurface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, SizeAndPosition.w, SizeAndPosition.h, 16, 0, 0, 0, 0);
	}

	SDL_Surface *tmp = SDL_DisplayFormat(ListBoxSurface);
	SDL_FreeSurface(ListBoxSurface);
	ListBoxSurface = tmp;

	// this is the destinatc_strion as needed for the new surface
	dest.x = 0;
	dest.y = 0;
	dest.w = ListBoxSurface->w;
	dest.h = ListBoxSurface->h;

	// Fill the surface with the correct color
	SDL_FillRect(ListBoxSurface, &dest, ListBoxColor);

	// this is the destination as needed for the new surface
	dest.x = 1;
	dest.y = 1;
	dest.w = ListBoxSurface->w-2;
	dest.h = SizeAndPosition.h-2;

	// Fill the surface with the correct color
	SDL_FillRect(ListBoxSurface, &dest, ListBoxBackgroundColor);

	if (Selected){
		// this is the destination as needed for the new surface
		dest.x = 1;
		dest.y = SizeAndPosition.h;
		dest.w = ListBoxSurface->w-2;
		dest.h = ListBoxSurface->h - 2 -SizeAndPosition.h;

		// Fill the surface with the correct color
		SDL_FillRect(ListBoxSurface, &dest, ListBoxBackgroundColor);
	}

	if ((unsigned) SizeAndPosition.h < (unsigned) ListBoxLabel.getHeight())
		return;

	if (SelectedIndex < List.size()){
		if ((unsigned)  SizeAndPosition.w-8 < (unsigned) ListBoxLabel.getWidth(List[SelectedIndex]))
			return;
	}

	if (Selected)
		Ypos = SizeAndPosition.h + (SizeAndPosition.h - ListBoxLabel.getHeight())/2;
	else
		Ypos = (SizeAndPosition.h - ListBoxLabel.getHeight())/2;

	if (SelectedIndex < List.size()){
		if (Selected){
			for (Uint32 i = 0; i< List.size(); i++){
				ListBoxLabel.Draw(List[i], ListBoxSurface, TextColor, 4, Ypos); //drawText("Your color", ListBox.geDrawingSurface(), 100, 20);
				Ypos += ListBoxLabel.getHeight()+Spacing;
			}
		}else{
			ListBoxLabel.Draw(List[SelectedIndex], ListBoxSurface, TextColor, 4, Ypos); //drawText("Your color", ListBox.geDrawingSurface(), 100, 20);
		}
	}
#ifndef USE_IMAGE_POOL
	if (ArrowDownImage != NULL)
		SDL_FreeSurface(ArrowDownImage);
#endif


if (!button_down){
#ifndef USE_IMAGE_POOL
	if ((ArrowDownImage = this->ReadShpImage ((char*)"btn-dn.shp", 0, 2)) == NULL){
#else
	if ((ArrowDownImage = pc::imgcache->getImage( ArrowDwn, 0 ).image) == NULL){
#endif
		printf ("%s line %i: Failed to load button down image\n", __FILE__, __LINE__);
	}
}else{
#ifndef USE_IMAGE_POOL
	if ((ArrowDownImage = this->ReadShpImage ((char*)"btn-dn.shp", 1, 2)) == NULL){
#else
	if ((ArrowDownImage = pc::imgcache->getImage( ArrowDwn, 1 ).image) == NULL){
#endif
		printf ("%s line %i: Failed to load button down image\n", __FILE__, __LINE__);
	}
}

if (ArrowDownImage != NULL){
	Uint8 ColRed;
	SDL_Color color;
	for (Uint32 x = 0; x < (Uint32) ArrowDownImage->w; x++){
		for (Uint32 y = 0; y < (Uint32) ArrowDownImage->h; y++){
		get_pixel ( ArrowDownImage, color, x, y );
		ColRed = color.r;
		color.r = (color.b+10)&0xff;
		if (color.g > 50)
		color.g = 50;
		color.b = ColRed-10;
		set_pixel (ArrowDownImage, color, x, y );
//		printf ("Col = %x\n", Temp[i]);
//		Temp[i] += 50;
		}
	}
}

// Fill the surface with the correct color
//ArrowDownImage = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, SizeAndPosition.w, SizeAndPosition.h, 16, 0, 0, 0, 0);
}

/* Function return true if the mouse is on top of the button */
bool DropDownListBox::MouseOver_button(void)
{
int mx, my;
int WinXpos = 0, WinYpos = 0;

	if (ArrowDownImage == NULL)
		return false;

	SDL_GetMouseState(&mx, &my);

	if (WindowToDrawOn != NULL){
		WindowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	if (mx > SizeAndPosition.x + SizeAndPosition.w - 1 && mx < SizeAndPosition.x + SizeAndPosition.w + ArrowDownImage->w - 1)
		if (my > SizeAndPosition.y && my < (SizeAndPosition.y + SizeAndPosition.h))
			return true;
	return false;
}

/* Function return true if the mouse is on top of the button */
bool DropDownListBox::MouseOver(void)
{
int mx, my;
int WinXpos = 0, WinYpos = 0;

	SDL_GetMouseState(&mx, &my);

	if (WindowToDrawOn != NULL){
		WindowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	Spacing = (SizeAndPosition.h - ListBoxLabel.getHeight())/2;

	if (mx > SizeAndPosition.x && mx < SizeAndPosition.x + SizeAndPosition.w){
		if (my > SizeAndPosition.y && my < (SizeAndPosition.y + (SizeAndPosition.h + Spacing + (SizeAndPosition.h - Spacing) * (signed)List.size())) && Selected)
			return true;
		if (my > SizeAndPosition.y && my < (SizeAndPosition.y + SizeAndPosition.h) && !Selected)
			return true;
	}
	return false;
}

/* Function returns the index (entry) the mouse is over */
Uint32 DropDownListBox::MouseOver_entry(void)
{
int mx, my;
int WinXpos = 0, WinYpos = 0;

	SDL_GetMouseState(&mx, &my);

	if (WindowToDrawOn != NULL){
		WindowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	if (!Selected)
		return 0;

	Spacing = (SizeAndPosition.h - ListBoxLabel.getHeight())/2;

	if (mx > SizeAndPosition.x && mx < SizeAndPosition.x + SizeAndPosition.w){
		if (my > (SizeAndPosition.y + SizeAndPosition.h) && my < (SizeAndPosition.y + (SizeAndPosition.h + Spacing + (SizeAndPosition.h - Spacing) * (signed)List.size()))){
			return  List.size() - (unsigned)(((SizeAndPosition.y + (SizeAndPosition.h + Spacing + (SizeAndPosition.h - Spacing) * List.size())) - my)/SizeAndPosition.h) - 1;
		}
	}
	return 0;
}

Uint32 DropDownListBox::selected(void)
{
	return SelectedIndex;
}

void DropDownListBox::HandleInput(SDL_Event event)
{
    //SDL_Event event;
    int mx, my;
    int WinXpos = 0, WinYpos = 0;

	SDL_GetMouseState(&mx, &my);

	if (WindowToDrawOn != NULL){
		WindowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	switch (event.type) {
		case SDL_MOUSEBUTTONDOWN:
			//printf ("%s line %i: Entry = %i\n", __FILE__, __LINE__, mouseover_entry());

			if (MouseOver_button())
				button_down = true;
			if (MouseOver() || MouseOver_button()){
				if (Selected){
					SelectedIndex = MouseOver_entry();
					Selected = false;
				}else{
					Selected = true;
				}
			}else{
				Selected = false;
			}
			Recreate = true;
			break;

		case SDL_MOUSEBUTTONUP:
			if (button_down){
				button_down = false;
				Recreate = true;
			}
			break;

		case SDL_KEYDOWN:
			if (!Selected)
				return;

			switch (event.key.keysym.sym){
				case SDLK_BACKSPACE:
				case SDLK_TAB:
				case SDLK_RSHIFT:
				case SDLK_LSHIFT:
					break;
				default:
					break;
			}
			break;
	}
}

bool DropDownListBox::Draw(int X, int Y)
{
    Uint32 Heigth = SizeAndPosition.h;
    SDL_Rect	dest;

	// Set size and posisiton
	SizeAndPosition.x = X;
	SizeAndPosition.y = Y;

	if (this->Recreate)
		Create();

	if (ListBoxSurface == NULL)
		return false;

	if (DisplaySurface == NULL)
		return false;

	if (WindowToDrawOn != NULL)
		SDL_BlitSurface(ListBoxSurface, NULL, WindowToDrawOn->GetWindowSurface (), &SizeAndPosition);
	else
		SDL_BlitSurface(ListBoxSurface, NULL, DisplaySurface, &SizeAndPosition);

	if ( ArrowDownImage != NULL ){

		dest.x = SizeAndPosition.x + SizeAndPosition.w - 1;
		dest.y = SizeAndPosition.y;
		dest.w = ArrowDownImage->w;
		dest.h = ArrowDownImage->h;

		if (WindowToDrawOn != NULL){
			SDL_BlitSurface(ArrowDownImage, NULL, WindowToDrawOn->GetWindowSurface (), &dest);
		}else{
			SDL_BlitSurface(ArrowDownImage, NULL, DisplaySurface, &dest);
		}
	}

	SizeAndPosition.h = Heigth;

	return true;
}

SliderControl::SliderControl()
{
//	SliderControl(100,90,false);
}

SliderControl::SliderControl(const int w, const int steps, const bool showSliderstatus)
{
	_windowToDrawOn	= NULL;
	_displaySurface	= NULL;
	_sliderSurface	= NULL;
	_position = 30;
	_oldPosition = 10;

	_sizeAndPosition.x = 0;
	_sizeAndPosition.y = 0;
	_sizeAndPosition.h = 16;
	_sizeAndPosition.w = w;

	// Initialize the surfaces
	_displaySurface          = pc::gfxeng->get_SDL_ScreenSurface();
	_sliderColor	    	    = SDL_MapRGB(_displaySurface->format, 0, 0, 0);
	_sliderBackgroundColor	    = SDL_MapRGB(_displaySurface->format, 255, 0, 0);
}

SliderControl::~SliderControl(void)
{
	if (_sliderSurface != NULL)
		SDL_FreeSurface(_sliderSurface);
}

// Set the surface (window) to draw the button on
void SliderControl::SetDrawingWindow(RA_WindowClass *Window)
{
	if (Window != NULL){
		_windowToDrawOn = Window;
	}
}

/* Function return true if the mouse is on top of the button */
bool SliderControl::MouseOver(void)
{
 int mx, my;
 int WinXpos = 0, WinYpos = 0;

	SDL_GetMouseState(&mx, &my);

	if (_windowToDrawOn != NULL){
		_windowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	if (mx > _sizeAndPosition.x && mx < _sizeAndPosition.x + _sizeAndPosition.w)
		if (my > _sizeAndPosition.y && my < _sizeAndPosition.y + _sizeAndPosition.h)
			return true;
	return false;
}

bool SliderControl::Draw(int X, int Y)
{
	// Set size and posisiton
	_sizeAndPosition.x = X;
	_sizeAndPosition.y = Y;

	if (_recreate)
		Create();

	if (_sliderSurface == NULL)
		return false;

	if (_displaySurface == NULL)
		return false;

	if (_windowToDrawOn != NULL)
		SDL_BlitSurface(_sliderSurface, NULL, _windowToDrawOn->GetWindowSurface (), &_sizeAndPosition);
	else
		SDL_BlitSurface(_sliderSurface, NULL, _displaySurface, &_sizeAndPosition);

	return true;
}

void SliderControl::CreateSlider (int x)
{
    SDL_Rect	dest;
	dest.x = x;
	dest.y = 0;
	dest.w = 30;
	dest.h = _sizeAndPosition.h;

	// Fill the surface with the correct color
	SDL_FillRect(_sliderSurface, &dest, _sliderColor);
}

void SliderControl::Create (void)
{
    SDL_Rect	dest;
//    Uint16		Ypos;
//    char		*temp;
	_recreate	= false;

	if (_sliderSurface != NULL){
		SDL_FreeSurface(_sliderSurface);
	}

	_sliderSurface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, _sizeAndPosition.w, _sizeAndPosition.h, 16, 0, 0, 0, 0);

	// this is the destinatc_strion as needed for the new surface
	dest.x = 0;
	dest.y = 0;
	dest.w = _sizeAndPosition.w;
	dest.h = _sizeAndPosition.h;

	// Fill the surface with the correct color
	SDL_FillRect(_sliderSurface, &dest, _sliderColor);

	// this is the destination as needed for the new surface
	dest.x = 1;
	dest.y = 1;
	dest.w = _sizeAndPosition.w-2;
	dest.h = _sizeAndPosition.h-2;

	// Fill the surface with the correct color
	SDL_FillRect(_sliderSurface, &dest, _sliderBackgroundColor);
        CreateSlider(_position);
}

void SliderControl::HandleSliding()
{
/*    if(_oldPosition >= _position && (_position <= _sizeAndPosition.x || _position-(_sizeAndPosition.w/_steps) <= _sizeAndPosition.x))
       _position -(_sizeAndPosition.w/_steps);//we are moving left
    else if(_oldPosition >= _position && (_position >= _sizeAndPosition.w || _position-(_sizeAndPosition.w/_steps) >= _sizeAndPosition.w))
       _position +(_sizeAndPosition.w/_steps);*/
       _position++;
       _recreate = true;
}

void SliderControl::HandleInput(SDL_Event event)
{
    //SDL_Event event;
    int mx, my;
    int WinXpos = 0, WinYpos = 0;

	SDL_GetMouseState(&mx, &my);

	if (_windowToDrawOn != NULL){
		_windowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	switch (event.type) {
		case SDL_MOUSEBUTTONDOWN:
			if (MouseOver()){
//                HandleSliding();
                  _position++;
                  _recreate = true;
				_selected = true;
			}else{
				_selected = false;
			}
			break;

		case SDL_MOUSEBUTTONUP:
			break;

		case SDL_KEYDOWN:
			if (!_selected)
				return;
				case SDLK_LSHIFT:
					break;
				default:
					break;
			break;
	}
}


RA_ProgressBar::RA_ProgressBar()
{
	_windowToDrawOn		= NULL;
	_displaySurface		= NULL;
	_progressSurface	= NULL;

	_sizeAndPosition.x = 0;
	_sizeAndPosition.y = 0;
	_sizeAndPosition.h = 13;
	_sizeAndPosition.w = 50;

	_recreate = true;
	_position = 0;
	_maxSteps = -1;
	_curStep = 0;


	// Initialize the surfaces
	_displaySurface			= pc::gfxeng->get_SDL_ScreenSurface();
	_progressBarColor		= SDL_MapRGB(_displaySurface->format, 172, 141, 49);
	_progressBackgroundColor	= SDL_MapRGB(_displaySurface->format, 98, 0, 0);
	_edgeLightColor			= SDL_MapRGB(_displaySurface->format, 205, 0, 0);
	_edgeDarkColor			= SDL_MapRGB(_displaySurface->format, 65, 0, 0);
}

RA_ProgressBar::~RA_ProgressBar()
{

}

void RA_ProgressBar::SetDrawingSurface (SDL_Surface *DwgSurface)
{
	if (_displaySurface != DwgSurface){
		_windowToDrawOn = NULL;
		_displaySurface = DwgSurface;
		_recreate = true;
	}
}

// 0 - 100%
void RA_ProgressBar::setProgressPosition (Uint8 Pos)
{
	if (Pos > 100)
		return;
	_position	= Pos;
	_recreate	= true;
}

// 0 - 100%
Uint8 RA_ProgressBar::getProgressPosition (void)
{
	return _position;
}

void RA_ProgressBar::setNumbSteps (int Steps)
{
	_maxSteps = Steps;
}

int RA_ProgressBar::getCurStep (void)
{
	return _curStep;
}

void RA_ProgressBar::setCurStep (Uint8 Step)
{
	_curStep = Step;

	Uint8 perc = _curStep * 100 / _maxSteps;

	if (perc > 100)
		perc = 100;

	// Set the percentage
	setProgressPosition (perc);
}

void RA_ProgressBar::SetDrawingWindow(RA_WindowClass *Window)
{
	if (Window != NULL){
		_windowToDrawOn = Window;
	}
}

bool RA_ProgressBar::MouseOver(void)
{
int mx, my;
int WinXpos = 0, WinYpos = 0;

	SDL_GetMouseState(&mx, &my);

	if (_windowToDrawOn != NULL){
		_windowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	if (mx > _sizeAndPosition.x && mx < _sizeAndPosition.x + _sizeAndPosition.w)
		if (my > _sizeAndPosition.y && my < _sizeAndPosition.y + _sizeAndPosition.h)
			return true;
	return false;
}

// Return true if the progress was updated
// Return false otherwise
bool RA_ProgressBar::HandleMouseClick(void)
{
int mx, my;
int WinXpos = 0, WinYpos = 0;
Uint8 perc;

	if (!MouseOver())
		return false;

	SDL_GetMouseState(&mx, &my);

	if (_windowToDrawOn != NULL){
		_windowToDrawOn->GetWindowPosition (&WinXpos, &WinYpos);
		mx -= WinXpos;
		my -= WinYpos;
	}

	// Calc tbe mouse x-pos relative of the slider pos
	mx -= _sizeAndPosition.x;

	//printf ("Percentage = %u\n", perc);
	if (_maxSteps == -1){
		// Cacl the percentage
		perc = mx * 100 / _sizeAndPosition.w;
	}else{
		_curStep = (Uint32) round ( (double)mx * (double)_maxSteps / (double)_sizeAndPosition.w);
		//printf ("Step = %i\n", Step);
		perc = _curStep * 100 / _maxSteps;
		//printf ("perc = %i\n", perc);
		if (perc > 100)
			perc = 100;
	}

	// Set the percentage
	setProgressPosition (perc);


	Draw(_sizeAndPosition.x, _sizeAndPosition.y);

	return true;
}


void RA_ProgressBar::Create (void)
{
SDL_Rect	dest;

	if (_progressSurface != NULL){
		SDL_FreeSurface(_progressSurface);
	}

	_progressSurface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, _sizeAndPosition.w, _sizeAndPosition.h, 16, 0, 0, 0, 0);

	// Draw the progress bar background
	dest.x = 0;
	dest.y = 0;
	dest.w = _sizeAndPosition.w;
	dest.h = _sizeAndPosition.h;
	SDL_FillRect(_progressSurface, &dest, _progressBackgroundColor);

	// Draw the actual progress bar
	dest.x = 0;
	dest.y = 0;
	dest.w = (_sizeAndPosition.w-4) * _position/100;
	dest.h = _sizeAndPosition.h;
	SDL_FillRect(_progressSurface, &dest, _progressBarColor);


	/// Start code for the progress bar edge

	// Draw the upper dark line
	dest.x = 0;
	dest.y = 0;
	dest.w = _sizeAndPosition.w;
	dest.h = 1;
	SDL_FillRect(_progressSurface, &dest, _edgeDarkColor);

	// Draw the lower light line
	dest.x = 0;
	dest.y = _sizeAndPosition.h - 1;
	dest.w = _sizeAndPosition.w;
	dest.h = 1;
	SDL_FillRect(_progressSurface, &dest, _edgeLightColor);

	// Draw the left dark line
	dest.x = 0;
	dest.y = 0;
	dest.w = 1;
	dest.h = _sizeAndPosition.h;
	SDL_FillRect(_progressSurface, &dest, _edgeDarkColor);

	// Draw the rigth light line
	dest.x = _sizeAndPosition.w - 1;
	dest.y = 0;
	dest.w = 1;
	dest.h = _sizeAndPosition.h;
	SDL_FillRect(_progressSurface, &dest, _edgeLightColor);

	/// Start code for drawing the position indicator

	// Draw the position left indicator line
	dest.x = (_sizeAndPosition.w-4) * _position/100;
	dest.y = 0;
	dest.w = 1;
	dest.h = _sizeAndPosition.h;
	SDL_FillRect(_progressSurface, &dest,_edgeLightColor);

	// Draw the position top ligth indicator line
	dest.x = (_sizeAndPosition.w-4) * _position/100;
	dest.y = 0;
	dest.w = 4;
	dest.h = 1;
	SDL_FillRect(_progressSurface, &dest,_edgeLightColor);


	// Draw the position rigth indicator line
	dest.x = (_sizeAndPosition.w-4) * _position/100 + 3;
	dest.y = 0;
	dest.w = 1;
	dest.h = _sizeAndPosition.h;
	SDL_FillRect(_progressSurface, &dest,_edgeDarkColor);

	// Draw the position bottum dark indicator line
	dest.x = (_sizeAndPosition.w-4) * _position/100;
	dest.y = _sizeAndPosition.h - 1;
	dest.w = 4;
	dest.h = 1;
	SDL_FillRect(_progressSurface, &dest,_edgeDarkColor);

	_recreate = false;

}

void RA_ProgressBar::Draw(int X, int Y)
{
	// Set size and posisiton
	_sizeAndPosition.x = X;
	_sizeAndPosition.y = Y;

	if (_recreate)
		Create();

	if (_progressSurface == NULL)
		return;

	if (_windowToDrawOn != NULL)
		SDL_BlitSurface(_progressSurface, NULL, _windowToDrawOn->GetWindowSurface (), &_sizeAndPosition);
	else
		SDL_BlitSurface(_progressSurface, NULL, _displaySurface, &_sizeAndPosition);

}

void RA_ProgressBar::Redraw ( void )
{
	Draw ( _sizeAndPosition.x, _sizeAndPosition.y);
}



